Package: cer
Title: Download and Tidy Australian Clean Energy Regulator Data
Version: 0.1.0
Authors@R: 
    person("Charles", "Coverdale", , "charlesfcoverdale@gmail.com", role = c("aut", "cre"))
Description: Fetch Australian Clean Energy Regulator data on carbon
    credits, safeguard mechanism facilities, renewable energy
    certificates, and greenhouse gas reporting. Provides tidy access
    to the Australian Carbon Credit Unit ('ACCU') Scheme project
    register, Safeguard Mechanism baselines and covered emissions,
    Large-scale Renewable Energy Target ('LRET') power station
    accreditations, Small-scale Renewable Energy Scheme ('SRES')
    installation data, the National Greenhouse and Energy Reporting
    ('NGER') scheme, and Quarterly Carbon Market Reports
    <https://cer.gov.au/markets/reports-and-data>. Includes a
    post-Chubb ACCU integrity layer (Chubb 2022 Independent Review),
    Safeguard reform handling (declining industry baselines from
    July 2023), National Greenhouse and Energy Reporting scope
    discipline (Scope 1 / Scope 2 market vs location / Climate
    Active), reconciliation against the Quarterly Carbon Market
    Report, and reproducibility helpers (snapshot pinning,
    SHA-256 cache integrity, session manifest, optional Zenodo
    deposit). Data is published by the Clean Energy Regulator
    under a Creative Commons Attribution 4.0 International licence.
Depends: R (>= 4.1.0)
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
Imports: cli (>= 3.6.0), httr2 (>= 1.0.0), readxl (>= 1.4.0), tools,
        utils
Suggests: digest, jsonlite, knitr, openssl, rmarkdown, testthat (>=
        3.0.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
URL: https://charlescoverdale.github.io/cer/,
        https://github.com/charlescoverdale/cer
BugReports: https://github.com/charlescoverdale/cer/issues
NeedsCompilation: no
Packaged: 2026-04-24 13:47:50 UTC; charlescoverdale
Author: Charles Coverdale [aut, cre]
Maintainer: Charles Coverdale <charlesfcoverdale@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-28 19:00:26 UTC
Built: R 4.6.0; ; 2026-05-01 21:29:42 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print,cer_tbl)
export(cer_accu_aggregate)
export(cer_accu_contracts)
export(cer_accu_issuances)
export(cer_accu_methods)
export(cer_accu_projects)
export(cer_accu_relinquishments)
export(cer_cache_info)
export(cer_cite)
export(cer_clear_cache)
export(cer_deposit_zenodo)
export(cer_international_comparator)
export(cer_legislative_instrument)
export(cer_lgc_power_stations)
export(cer_manifest)
export(cer_manifest_clear)
export(cer_manifest_write)
export(cer_method_determination)
export(cer_method_integrity)
export(cer_nger_climate_active)
export(cer_nger_corporate)
export(cer_nger_electricity)
export(cer_nger_scope)
export(cer_qcmr)
export(cer_qcmr_reference)
export(cer_reconcile)
export(cer_safeguard_baseline_trajectory)
export(cer_safeguard_facilities)
export(cer_safeguard_industry_baselines)
export(cer_safeguard_teba_facilities)
export(cer_sha256)
export(cer_snapshot)
export(cer_sres_installations)
export(cer_to_carbondata)

# CharacTER

CharacTER: Translation Edit Rate on Character Level

CharacTer is a novel character level metric inspired by the commonly applied translation edit rate (TER). It is defined
as the minimum number of character edits required to adjust a hypothesis, until it completely matches the reference,
normalized by the length of the hypothesis sentence. CharacTer calculates the character level edit distance while
performing the shift edit on word level. Unlike the strict matching criterion in Ter, a hypothesis word is considered
to match a reference word and could be shifted, if the edit distance between them is below a threshold value. The
Levenshtein distance between the reference and the shifted hypothesis sequence is computed on the character level. In
addition, the lengths of hypothesis sequences instead of reference sequences are used for normalizing the edit
distance, which effectively counters the issue that shorter translations normally achieve lower TER.


## Modifications by Bram Vanroy

Bram Vanroy made some changes to this package that does not effect the result of calculate the metric but that should
improve usability. Code has been re-written to avoid the need for custom C++ code (instead the [C implementation
of Levenshtein](https://github.com/maxbachmann/Levenshtein) alongside an LRU cache is used), to make functions more
accessible and readable, and typing info has been included. Packaging has also improved to make uploading to PyPi a
breeze. This means that the package can now be installed via pip:

```shell
pip install cer
```

The main functions are `calculate_cer` and `calculate_cer_corpus`, which both expect tokenized input. The first
argument contains the hypotheses and the second the references.

```python
from cer import calculate_cer

cer_score = calculate_cer(["i", "like", "your", "bag"], ["i", "like", "their", "bags"])
cer_score
0.3333333333333333
```

`calculate_cer_corpus` is similar but instead it expects a sequence of sequence of words, basically a corpus of
sentences of words. It will report some statistics of the sentence-level CER scores that were calculated.

```python
from cer import calculate_cer_corpus

hyps = ["this week the saudis denied information published in the new york times",
        "this is in fact an estimate"]
refs = ["saudi arabia denied this week information published in the american new york times",
        "this is actually an estimate"]

hyps = [sent.split() for sent in hyps]
refs = [sent.split() for sent in refs]

cer_corpus_score = calculate_cer_corpus(hyps, refs)
cer_corpus_score
{
    'count': 2,
    'mean': 0.3127282211789254,
    'median': 0.3127282211789254,
    'std': 0.07561653111280243,
    'min': 0.25925925925925924,
    'max': 0.36619718309859156
}
```

In addition to the Python interface, a command-line entry-point is also installed, which you can use as
`calculate-cer`. Its idea is to calculate aggregate scores on the corpus-level (similar to calculate_cer_corpus)
based on two input files. One with hypotheses and one with references (one on each line). Results are written to
stdout.

```shell
usage: calculate-cer [-h] [-r] fhyp fref

CharacTER: Character Level Translation Edit Rate

positional arguments:
  fhyp                Path to file containing hypothesis sentences. One per line.
  fref                Path to file containing reference sentences. One per line.

optional arguments:
  -h, --help          show this help message and exit
  -r, --per_sentence  Whether to output CER scores per ref/hyp pair in addition to corpus-level statistics
```

## License
[GPLv3](LICENSE)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", eval = FALSE)

## -----------------------------------------------------------------------------
# library(cer)
# cer_snapshot("2026-04-24")
# 
# corp <- cer_nger_corporate(year = 2025)
# 
# scope1 <- cer_nger_scope(corp, scope = "1")
# head(scope1[, c("reporting_corporation", "emissions_t_co2e", "scope")])
# 
# scope12 <- cer_nger_scope(corp, scope = "1_plus_2")
# sum(scope12$emissions_t_co2e, na.rm = TRUE)

## -----------------------------------------------------------------------------
# ca <- cer_nger_climate_active(corp)
# ca_cols <- grep("operational_scope", names(ca), value = TRUE)
# head(ca[, c("reporting_corporation", ca_cols)])

## -----------------------------------------------------------------------------
# elec_fac <- cer_nger_electricity(year = 2025)
# 
# # Electricity subset at corporate level (heuristic: ANZSIC 26 or
# # a "generation" substring).
# elec_corp <- corp[grepl("electric|power|generat",
#                          corp$reporting_corporation,
#                          ignore.case = TRUE), ]
# 
# fac_total <- sum(elec_fac$emissions_t_co2e, na.rm = TRUE)
# corp_total <- sum(elec_corp$scope_1_emissions_t_co2e, na.rm = TRUE)
# 
# # Gap is informative: mismatch suggests corporate/facility
# # attribution differs or NGER updated one but not the other.
# pct_gap <- (fac_total - corp_total) / corp_total
# pct_gap

# NGER scope discipline helpers.
#
# Scope confusion is the most common error in Australian emissions
# reporting. NGER publishes Scope 1 + Scope 2 at corporate level,
# but only Scope 1 at facility level (for the electricity sector).
# Climate Active, TCFD, and SBTi require scope-explicit reporting.

#' Select NGER columns by scope with explicit errors
#'
#' Given an NGER corporate-level data frame from
#' [cer_nger_corporate()], select the requested scope columns and
#' error if they are not present. Avoids the silent-misaggregation
#' failure mode where a user sums a column that is actually Scope
#' 1+2 combined when they intended Scope 1 only.
#'
#' Column-name detection is heuristic because NGER renames columns
#' year to year. Pass `col_pattern` to override.
#'
#' @param x A `cer_tbl` or data frame from [cer_nger_corporate()].
#' @param scope One of `"1_plus_2"` (default), `"1"`, `"2_market"`,
#'   or `"2_location"`.
#' @param col_pattern Optional regex override for the target column.
#'
#' @return A data frame with an `emissions_t_co2e` column populated
#'   by the requested scope and a `scope` column recording which
#'   scope was selected.
#' @family nger
#' @export
#' @examples
#' \donttest{
#' op <- options(cer.cache_dir = tempdir())
#' try({
#'   x <- cer_nger_corporate(2025)
#'   cer_nger_scope(x, scope = "1")
#' })
#' options(op)
#' }
cer_nger_scope <- function(x,
                            scope = c("1_plus_2", "1",
                                      "2_market", "2_location"),
                            col_pattern = NULL) {
  scope <- match.arg(scope)
  df <- as.data.frame(x, stringsAsFactors = FALSE)
  if (is.null(col_pattern)) {
    col_pattern <- switch(scope,
      `1_plus_2`    = "scope.?1.?(and|[+]).?2|scope_1_2|total.?emissions",
      `1`           = "scope.?1(?!.?2)",
      `2_market`    = "scope.?2.?market|scope_2_market",
      `2_location`  = "scope.?2.?location|scope_2_location|scope.?2(?!.?market)"
    )
  }
  # Try Perl-regex; fall back if cer renames don't use the markers.
  hit <- suppressWarnings(
    grep(col_pattern, names(df), perl = TRUE, ignore.case = TRUE,
         value = TRUE)
  )
  if (length(hit) == 0L) {
    cli::cli_abort(c(
      "Could not find a column for scope {.val {scope}}.",
      "i" = "NGER column names drift year-to-year; pass {.arg col_pattern}.",
      "i" = "Available columns: {.val {head(names(df), 20L)}}"
    ))
  }
  df$emissions_t_co2e <- suppressWarnings(as.numeric(df[[hit[1L]]]))
  df$scope <- scope
  df$scope_source_column <- hit[1L]
  df
}

#' Translate NGER scope columns to Climate Active categories
#'
#' Climate Active (Australia's voluntary carbon-neutral certification)
#' uses a Scope 1 / Scope 2 (market-based and location-based) /
#' Scope 3 taxonomy. NGER publishes Scope 1 and Scope 2 only (Scope 3
#' is not NGER-reportable). This helper renames columns so an NGER
#' output flows into a Climate Active inventory template.
#'
#' @param x A `cer_tbl` or data frame from [cer_nger_corporate()].
#'
#' @return A data frame with Climate Active-style columns:
#'   `operational_scope_1_t_co2e`, `operational_scope_2_location_t_co2e`,
#'   `operational_scope_2_market_t_co2e`.
#'
#' @references
#' Department of Climate Change, Energy, the Environment and Water
#'   (annual). \emph{Climate Active Carbon Neutral Standard for
#'   Organisations}. Commonwealth of Australia.
#'
#' Greenhouse Gas Protocol (2004). \emph{A Corporate Accounting and
#'   Reporting Standard (Revised Edition)}. World Resources Institute.
#'
#' @family nger
#' @export
#' @examples
#' \donttest{
#' op <- options(cer.cache_dir = tempdir())
#' try({
#'   x <- cer_nger_corporate(2025)
#'   cer_nger_climate_active(x)
#' })
#' options(op)
#' }
cer_nger_climate_active <- function(x) {
  df <- as.data.frame(x, stringsAsFactors = FALSE)

  rename_if <- function(df, old, new) {
    i <- which(names(df) == old)
    if (length(i) > 0L) names(df)[i] <- new
    df
  }

  # Attempt heuristic rename on the most common NGER column shapes.
  scope1_col  <- grep("^scope_1(?!.*2)", names(df), perl = TRUE, value = TRUE)
  scope2m_col <- grep("scope.?2.?market",   names(df), perl = TRUE, value = TRUE)
  scope2l_col <- grep("scope.?2.?location", names(df), perl = TRUE, value = TRUE)

  if (length(scope1_col) > 0L) {
    df <- rename_if(df, scope1_col[1L], "operational_scope_1_t_co2e")
  }
  if (length(scope2m_col) > 0L) {
    df <- rename_if(df, scope2m_col[1L], "operational_scope_2_market_t_co2e")
  }
  if (length(scope2l_col) > 0L) {
    df <- rename_if(df, scope2l_col[1L], "operational_scope_2_location_t_co2e")
  }
  df$climate_active_ready <- TRUE
  df
}

# Quarterly Carbon Market Report

#' @noRd
QCMR_LANDING <- "https://cer.gov.au/markets/reports-and-data/quarterly-carbon-market-reports"

#' @noRd
qcmr_url <- function(quarter) {
  if (identical(quarter, "latest")) {
    hits <- cer_scrape_links(QCMR_LANDING, "qcmr.*data.*workbook")
    if (length(hits) == 0L) {
      cli::cli_abort(c(
        "Could not find any QCMR data workbook on the landing page.",
        "i" = "Browse {.url {QCMR_LANDING}} manually."
      ))
    }
    return(hits[1L])
  }

  q <- tolower(gsub("[^0-9a-z]", "", quarter))
  parts <- regmatches(q, regexec("^([0-9]{4})q([1-4])$", q))[[1]]
  if (length(parts) != 3L) {
    cli::cli_abort(c(
      "Could not parse quarter {.val {quarter}}.",
      "i" = "Use format like {.val 2025q4} or {.val 2024q2}, or pass {.val latest}."
    ))
  }
  yr <- parts[2]
  qn <- as.integer(parts[3])
  month <- c("march", "june", "september", "december")[qn]
  pattern <- sprintf("qcmr.*%s.*%s|qcmr.*%s.*%s", month, yr, yr, month)
  hits <- cer_scrape_links(QCMR_LANDING, pattern)
  if (length(hits) == 0L) {
    cli::cli_abort(c(
      "No QCMR data workbook found for {month} quarter {yr}.",
      "i" = "Try {.val latest} or browse {.url {QCMR_LANDING}}."
    ))
  }
  hits[1L]
}

#' Quarterly Carbon Market Report data workbook
#'
#' Aggregate carbon market statistics from the CER's Quarterly
#' Carbon Market Report, including ACCU issuances, LGC creations,
#' STC volumes, unit surrenders, and Safeguard Mechanism
#' developments.
#'
#' @param quarter Character. Quarter identifier in `"YYYYqN"` form
#'   (e.g. `"2025q4"` for the December 2025 quarter). Defaults to
#'   `"latest"`, which resolves to the most recent release.
#' @param sheet Sheet name or integer index. The CER QCMR workbook
#'   contains a `Contents` sheet (default) plus one sheet per
#'   figure (e.g. `"Figure 1.1"`, `"Figure 3.14"`). Pass a figure
#'   name to extract that figure's data.
#'
#' @return A `cer_tbl` with the requested sheet's contents.
#'
#' @source Clean Energy Regulator, Quarterly Carbon Market Reports:
#'   <https://cer.gov.au/markets/reports-and-data/quarterly-carbon-market-reports>.
#'   Licensed under CC BY 4.0.
#'
#' @references
#' Clean Energy Regulator (quarterly). \emph{Quarterly Carbon
#'   Market Report}. Methodology notes accompany each release.
#'
#' Clean Energy Regulator (2024). \emph{Carbon Market Indicators:
#'   methodology and data dictionary}.
#'
#' @family qcmr
#' @export
#' @examples
#' \donttest{
#' op <- options(cer.cache_dir = tempdir())
#' try({
#'   # Contents sheet lists all available figures
#'   toc <- cer_qcmr("latest")
#'   head(toc)
#'   # Fetch a specific figure
#'   fig <- cer_qcmr("latest", sheet = "Figure 1.1")
#' })
#' options(op)
#' }
cer_qcmr <- function(quarter = "latest", sheet = "Contents") {
  url <- qcmr_url(quarter)
  df <- cer_fetch_auto(url, sheet = sheet)
  rownames(df) <- NULL
  new_cer_tbl(df,
              source = QCMR_LANDING,
              licence = "CC BY 4.0",
              title = paste0("QCMR ", quarter))
}

test_that("qcmr_url rejects malformed quarter strings", {
  skip_if_offline()
  expect_error(cer:::qcmr_url("2025"))
  expect_error(cer:::qcmr_url("2025q5"))
  expect_error(cer:::qcmr_url("Q4 2025"))
})

test_that("cer_qcmr live fetch", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("CER_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")

  x <- cer_qcmr("latest")
  expect_s3_class(x, "cer_tbl")
  expect_gt(nrow(x), 0L)
})

# Package-level environment for session state (manifest registry,
# snapshot pin). Created at load time by .onLoad().

#' @noRd
.cer_env <- new.env(parent = emptyenv())

.onLoad <- function(libname, pkgname) {
  .cer_env$manifest <- list()
  invisible(NULL)
}

# ACCU integrity layer : the unique `cer` contribution.
#
# After Chubb (2022), Australian carbon credit units cannot be
# treated as fungible. Different methods have different integrity
# profiles; some are under review; some are suspended. This layer
# surfaces those distinctions to users before they aggregate or
# publish numbers.

#' ACCU method integrity scorecard
#'
#' Returns a method-level integrity scorecard for all ACCU methods.
#' Columns include `integrity_tier` (high / standard / contested),
#' `chubb_affected` (whether the Chubb 2022 Independent Review
#' flagged the method), `chubb_recommendation_applies` (whether a
#' specific Chubb recommendation targets the method), and
#' `federal_register_instrument` (authoritative legislative
#' instrument ID).
#'
#' Use this to filter or weight ACCU aggregates, or to annotate a
#' chart with integrity context. For example, an analyst reporting
#' "185 million ACCUs issued to date" should, at minimum, break
#' the total down by `integrity_tier` and flag the contested share.
#'
#' @param tier Optional filter on `integrity_tier`. One or more of
#'   `"high"`, `"standard"`, `"contested"`.
#' @param status Optional filter on method `status`. One or more of
#'   `"active"`, `"suspended"`, `"under_review"`, `"superseded"`,
#'   `"expired"`.
#'
#' @return A `cer_tbl` with one row per method (19 rows at v0.2.0).
#'
#' @references
#' Chubb, I. (2022). \emph{Independent Review of Australian Carbon
#'   Credit Units}. Commonwealth of Australia, Department of
#'   Climate Change, Energy, the Environment and Water.
#'
#' Macintosh, A., Butler, D., Larraondo, P., Evans, M.C., Ansell,
#'   D., Gibbons, P., Lindenmayer, D., Waschka, M. and Fisher, R.
#'   (2022). "The emperor's new clothes: assessing the integrity
#'   of ACCUs." Australian National University working paper series.
#'
#' Butler, D., Macintosh, A. and Pouliot, M. (2023). "Response to
#'   the Chubb Review." Australian National University.
#'
#' Ansell, D. \emph{et al.} (2020). "Contemporary fire regimes of
#'   northern Australia: a review of current patterns, impacts and
#'   challenges." \emph{Environmental Research Reviews}.
#'
#' @family accu
#' @export
#' @examples
#' cer_method_integrity()
#' cer_method_integrity(tier = "contested")
#' cer_method_integrity(status = "active")
cer_method_integrity <- function(tier = NULL, status = NULL) {
  path <- system.file("extdata", "accu_method_integrity.csv",
                       package = "cer")
  if (!nzchar(path) || !file.exists(path)) {
    cli::cli_abort(
      "Method integrity CSV not installed at {.path inst/extdata/}."
    )
  }
  df <- utils::read.csv(path, stringsAsFactors = FALSE,
                         na.strings = c("", "NA"), check.names = FALSE)
  if (!is.null(tier)) {
    df <- df[df$integrity_tier %in% tolower(tier), , drop = FALSE]
  }
  if (!is.null(status)) {
    df <- df[df$status %in% tolower(status), , drop = FALSE]
  }
  rownames(df) <- NULL
  new_cer_tbl(df,
              source = "https://cer.gov.au/schemes/australian-carbon-credit-unit-scheme/accu-scheme-methods",
              licence = "CC BY 4.0",
              title = "ACCU method integrity scorecard (post-Chubb)")
}

#' Federal Register URL for an ACCU method determination
#'
#' Given a method short name (e.g. `"HIR"`, `"Savanna_2026"`),
#' return the authoritative legislative instrument URL on the
#' Federal Register of Legislation. This is the technical
#' determination document that defines measurement, monitoring,
#' baseline, and permanence rules for the method.
#'
#' @param method_short Character scalar matching a `method_short`
#'   value from [cer_method_integrity()].
#'
#' @return A length-1 character URL, or `NA_character_` if not found.
#' @family accu
#' @export
#' @examples
#' cer_method_determination("HIR")
#' cer_method_determination("Savanna_2026")
cer_method_determination <- function(method_short) {
  stopifnot(is.character(method_short), length(method_short) == 1L)
  df <- as.data.frame(cer_method_integrity())
  hit <- df[df$method_short == method_short, ]
  if (nrow(hit) == 0L) {
    cli::cli_warn(c(
      "No method found with {.val method_short = {method_short}}.",
      "i" = "Run {.code cer_method_integrity()} to see available names."
    ))
    return(NA_character_)
  }
  hit$latest_determination_url[1L]
}

#' Integrity-weighted ACCU aggregation
#'
#' Takes an `ato_tbl` or data frame of ACCU project records (from
#' [cer_accu_projects()]), joins the method integrity scorecard,
#' and aggregates by integrity tier. Emits a warning if contested
#' or under-review methods contribute materially to the total so
#' the analyst cannot silently aggregate them into a single
#' headline figure.
#'
#' @param x A `cer_tbl` or data frame with columns `method` and
#'   (typically) `accus_issued`.
#' @param value_col Column to sum. Default `"accus_issued"`.
#' @param warn_contested_pct Warn if the contested tier exceeds
#'   this share of the total. Default `0.05` (5%).
#'
#' @return A data frame with one row per integrity tier and
#'   columns `integrity_tier`, `projects`, `accus_issued_sum`,
#'   `share_pct`.
#' @family accu
#' @export
#' @examples
#' \donttest{
#' op <- options(cer.cache_dir = tempdir())
#' try({
#'   p <- cer_accu_projects()
#'   cer_accu_aggregate(p)
#' })
#' options(op)
#' }
cer_accu_aggregate <- function(x,
                                value_col = "accus_issued",
                                warn_contested_pct = 0.05) {
  if (is.data.frame(x)) {
    df <- as.data.frame(x, stringsAsFactors = FALSE)
  } else {
    cli::cli_abort("{.arg x} must be a data frame.")
  }
  if (!"method" %in% names(df)) {
    cli::cli_abort(
      "{.arg x} has no {.val method} column."
    )
  }
  if (!value_col %in% names(df)) {
    cli::cli_warn(c(
      "Column {.val {value_col}} not in {.arg x}; using row counts.",
      "i" = "Pass {.arg value_col} to specify a numeric column."
    ))
    df$.value <- 1
  } else {
    df$.value <- suppressWarnings(as.numeric(df[[value_col]]))
  }

  integ <- as.data.frame(cer_method_integrity())
  df$method_lower <- tolower(df$method)
  integ$method_lower <- tolower(integ$method_full_name)

  joined <- merge(df, integ[, c("method_lower", "integrity_tier")],
                   by = "method_lower", all.x = TRUE)
  joined$integrity_tier[is.na(joined$integrity_tier)] <- "unclassified"

  agg <- stats::aggregate(
    cbind(projects = 1, value = joined$.value) ~ integrity_tier,
    data = joined, FUN = sum, na.rm = TRUE
  )
  names(agg)[names(agg) == "value"] <- paste0(value_col, "_sum")
  total <- sum(agg[[paste0(value_col, "_sum")]], na.rm = TRUE)
  agg$share_pct <- 100 * agg[[paste0(value_col, "_sum")]] / total
  agg <- agg[order(-agg$share_pct), , drop = FALSE]
  rownames(agg) <- NULL

  contested_share <- sum(agg[[paste0(value_col, "_sum")]][
    agg$integrity_tier == "contested"
  ], na.rm = TRUE) / total
  if (isTRUE(contested_share > warn_contested_pct)) {
    cli::cli_warn(c(
      "Contested-integrity methods account for {round(100 * contested_share, 1)}% of the total.",
      "i" = "Chubb (2022) flagged these methods for review.",
      "i" = "Do not report a single headline figure without breaking down by integrity tier."
    ))
  }
  agg
}

#' @noRd
# Internal helper: attach a `chubb_flag` column to a data frame of
# ACCU projects, looking up the method via the integrity scorecard.
cer_attach_chubb_flag <- function(df) {
  if (!"method" %in% names(df)) return(df)
  integ <- as.data.frame(cer_method_integrity())
  integ$method_lower <- tolower(integ$method_full_name)
  df$method_lower <- tolower(df$method)
  lookup <- stats::setNames(integ$status, integ$method_lower)
  df$chubb_flag <- unname(lookup[df$method_lower])
  df$chubb_flag[is.na(df$chubb_flag)] <- "unclassified"
  df$method_lower <- NULL
  df
}

test_that("cer_method_integrity returns a data frame", {
  m <- cer_method_integrity()
  expect_s3_class(m, "data.frame")
  expect_gt(nrow(m), 15L)
  expect_true(all(c("method_short", "status", "integrity_tier",
                    "chubb_affected") %in% names(m)))
})

test_that("cer_method_integrity tier filter works", {
  contested <- cer_method_integrity(tier = "contested")
  expect_true(all(contested$integrity_tier == "contested"))
  expect_gt(nrow(contested), 0L)
})

test_that("cer_method_integrity status filter works", {
  suspended <- cer_method_integrity(status = "suspended")
  expect_true(all(suspended$status == "suspended"))
})

# Level 2: HIR should be flagged under_review (Chubb recommendation).
test_that("HIR is under_review post-Chubb", {
  m <- cer_method_integrity()
  hir <- m[m$method_short == "HIR", ]
  expect_equal(hir$status, "under_review")
  expect_true(hir$chubb_affected)
})

# Level 2: Avoided Deforestation suspended post-Chubb.
test_that("AD is suspended post-Chubb", {
  m <- cer_method_integrity()
  ad <- m[m$method_short == "AD", ]
  expect_equal(ad$status, "suspended")
})

test_that("cer_method_determination returns Federal Register URL", {
  url <- cer_method_determination("HIR")
  expect_match(url, "legislation\\.gov\\.au")
})

test_that("cer_method_determination warns on unknown method", {
  expect_warning(cer_method_determination("NOT_A_METHOD"), "No method")
})

# Level 3: aggregate is shape-stable across tiers.
test_that("cer_accu_aggregate returns tier breakdown", {
  fake <- data.frame(
    method = c("Human-Induced Regeneration of a permanent even-aged native forest",
               "Savanna Fire Management 2026",
               "Landfill Gas Capture and Combustion"),
    accus_issued = c(100000, 50000, 30000),
    stringsAsFactors = FALSE
  )
  agg <- suppressWarnings(cer_accu_aggregate(fake))
  expect_s3_class(agg, "data.frame")
  expect_true("integrity_tier" %in% names(agg))
  expect_true("share_pct" %in% names(agg))
  expect_equal(sum(agg$share_pct), 100, tolerance = 0.01)
})

# Level 3: aggregate warns when contested > 5%.
test_that("cer_accu_aggregate warns on high contested share", {
  fake <- data.frame(
    method = c("Human-Induced Regeneration of a permanent even-aged native forest"),
    accus_issued = 1000000,
    stringsAsFactors = FALSE
  )
  # HIR is "contested" tier; 100% of the total triggers the warn.
  expect_warning(cer_accu_aggregate(fake), "Contested-integrity")
})

# Cache management

#' @noRd
cer_cache_dir <- function() {
  d <- getOption("cer.cache_dir",
                 default = tools::R_user_dir("cer", "cache"))
  if (!dir.exists(d)) dir.create(d, recursive = TRUE)
  d
}

#' Clear the cer cache
#'
#' Deletes all locally cached files. The next call to any data
#' function will re-download from the Clean Energy Regulator.
#'
#' @return Invisibly returns `NULL`.
#' @family configuration
#' @export
#' @examples
#' \donttest{
#' op <- options(cer.cache_dir = tempdir())
#' cer_clear_cache()
#' options(op)
#' }
cer_clear_cache <- function() {
  d <- cer_cache_dir()
  files <- list.files(d, full.names = TRUE)
  n <- length(files)
  if (n > 0L) unlink(files, recursive = TRUE)
  cli::cli_inform("Removed {n} cached file{?s} from {.path {d}}.")
  invisible(NULL)
}

#' Inspect the local cer cache
#'
#' Returns the cache directory, number of files, total size, and
#' a per-file summary. Useful for deciding whether to call
#' [cer_clear_cache()].
#'
#' @return A list with elements `dir`, `n_files`, `size_bytes`,
#'   `size_human`, and `files` (a data frame with `name`,
#'   `size_bytes`, `modified`).
#' @family configuration
#' @export
#' @examples
#' \donttest{
#' op <- options(cer.cache_dir = tempdir())
#' cer_cache_info()
#' options(op)
#' }
cer_cache_info <- function() {
  d <- cer_cache_dir()
  empty <- data.frame(
    name = character(0L),
    size_bytes = numeric(0L),
    modified = as.POSIXct(character(0L)),
    stringsAsFactors = FALSE
  )
  paths <- list.files(d, full.names = TRUE)
  if (length(paths) == 0L) {
    return(list(dir = d, n_files = 0L, size_bytes = 0,
                size_human = "0 B", files = empty))
  }
  info <- file.info(paths)
  files <- data.frame(
    name = basename(paths),
    size_bytes = info$size,
    modified = info$mtime,
    stringsAsFactors = FALSE
  )
  files <- files[order(-files$size_bytes), , drop = FALSE]
  rownames(files) <- NULL
  total <- sum(files$size_bytes)
  list(dir = d, n_files = nrow(files), size_bytes = total,
       size_human = cer_format_bytes(total), files = files)
}

test_that("cer_cache_dir honours the cer.cache_dir option", {
  tmp <- tempfile("cer_cache_")
  op <- options(cer.cache_dir = tmp)
  on.exit(options(op), add = TRUE)

  d <- cer:::cer_cache_dir()
  expect_equal(normalizePath(d), normalizePath(tmp))
  expect_true(dir.exists(d))
})

test_that("cer_cache_info returns the expected structure on an empty cache", {
  tmp <- tempfile("cer_cache_")
  op <- options(cer.cache_dir = tmp)
  on.exit(options(op), add = TRUE)

  info <- cer_cache_info()
  expect_type(info, "list")
  expect_setequal(names(info),
                  c("dir", "n_files", "size_bytes", "size_human", "files"))
  expect_equal(info$n_files, 0L)
  expect_equal(info$size_bytes, 0)
  expect_s3_class(info$files, "data.frame")
  expect_equal(nrow(info$files), 0L)
})

test_that("cer_cache_info counts files correctly", {
  tmp <- tempfile("cer_cache_")
  op <- options(cer.cache_dir = tmp)
  on.exit(options(op), add = TRUE)

  dir.create(tmp, recursive = TRUE)
  writeLines("a", file.path(tmp, "a.csv"))
  writeLines("ab", file.path(tmp, "b.csv"))

  info <- cer_cache_info()
  expect_equal(info$n_files, 2L)
  expect_gte(info$size_bytes, 3)
  expect_match(info$size_human, "B$")
})

test_that("cer_clear_cache removes cached files silently", {
  tmp <- tempfile("cer_cache_")
  op <- options(cer.cache_dir = tmp)
  on.exit(options(op), add = TRUE)

  dir.create(tmp, recursive = TRUE)
  writeLines("x", file.path(tmp, "x.csv"))
  expect_true(file.exists(file.path(tmp, "x.csv")))

  expect_invisible(cer_clear_cache())
  expect_false(file.exists(file.path(tmp, "x.csv")))
})

test_that("cer_format_bytes formats across SI thresholds", {
  expect_equal(cer:::cer_format_bytes(0), "0 B")
  expect_equal(cer:::cer_format_bytes(500), "500 B")
  expect_match(cer:::cer_format_bytes(1500), "KB$")
  expect_match(cer:::cer_format_bytes(1500000), "MB$")
  expect_match(cer:::cer_format_bytes(1500000000), "GB$")
})

# Internal helpers

#' @noRd
`%||%` <- function(x, y) if (is.null(x)) y else x

#' @noRd
cer_format_bytes <- function(x) {
  if (is.na(x) || x < 1024) return(paste0(x, " B"))
  units <- c("KB", "MB", "GB", "TB")
  for (i in seq_along(units)) {
    x <- x / 1024
    if (x < 1024 || i == length(units)) {
      return(sprintf("%.1f %s", x, units[i]))
    }
  }
}

#' Clean column names: lowercase, snake_case, strip punctuation
#' @param x Character vector of column names.
#' @return Character vector of cleaned names.
#' @noRd
cer_clean_names <- function(x) {
  x <- tolower(x)
  x <- gsub("[^a-z0-9]+", "_", x)
  x <- gsub("^_+|_+$", "", x)
  x <- gsub("_+", "_", x)
  x
}

#' Fetch a CSV from the CER site and return a tidy data frame
#'
#' Uses the cache-aware downloader to retrieve the file, then
#' \code{utils::read.csv} with sensible defaults. Column names
#' are cleaned to snake_case.
#'
#' @param url URL to a CSV file.
#' @param ... Passed to \code{utils::read.csv}.
#' @return A data frame.
#' @noRd
cer_fetch_csv <- function(url, ...) {
  file <- cer_download_cached(url)
  df <- utils::read.csv(file, stringsAsFactors = FALSE,
                        check.names = FALSE,
                        na.strings = c("", "NA", "N/A", "-"),
                        ...)
  names(df) <- cer_clean_names(names(df))
  df
}

#' Fetch an XLSX from the CER site, returning a data frame
#'
#' @param url URL to an XLSX file.
#' @param sheet Sheet name or index.
#' @param skip Number of rows to skip.
#' @return A data frame.
#' @noRd
cer_fetch_xlsx <- function(url, sheet = 1, skip = 0) {
  if (!requireNamespace("readxl", quietly = TRUE)) {
    cli::cli_abort(c(
      "The {.pkg readxl} package is required to parse XLSX files.",
      "i" = "Install it with {.code install.packages('readxl')}."
    ))
  }
  file <- cer_download_cached(url)
  df <- as.data.frame(
    readxl::read_excel(file, sheet = sheet, skip = skip, .name_repair = "minimal"),
    stringsAsFactors = FALSE
  )
  names(df) <- cer_clean_names(names(df))
  df
}

#' Fetch and parse a CER document URL, auto-detecting CSV vs XLSX
#'
#' CER's `/document/<slug>` and `/document/<slug>-0` pairs follow
#' inconsistent conventions: ACCU has XLSX at the plain slug and
#' CSV at `-0`, while Safeguard has CSV at the plain slug and XLSX
#' at `-0`. Rather than guessing from the URL, this helper
#' downloads the file and inspects the first bytes: XLSX is a ZIP
#' container starting with the `PK\x03\x04` signature.
#'
#' For XLSX files, many CER workbooks have a multi-line title or
#' narrative block before the data header. The helper reads the
#' first 15 rows without a header, finds the first row where at
#' least three cells are non-empty character strings (the header
#' row), then re-reads with that row as the header.
#'
#' @param url URL to a CER data document.
#' @param sheet Sheet name or index (used only for XLSX).
#' @return A data frame.
#' @noRd
cer_fetch_auto <- function(url, sheet = 1) {
  file <- cer_download_cached(url)
  sig <- tryCatch(readBin(file, what = "raw", n = 4L),
                  error = function(e) raw(0))
  is_xlsx <- length(sig) >= 4L &&
    identical(sig[1:2], as.raw(c(0x50, 0x4B)))  # "PK" (zip magic)

  if (is_xlsx) {
    if (!requireNamespace("readxl", quietly = TRUE)) {
      cli::cli_abort(c(
        "The {.pkg readxl} package is required to parse XLSX files."
      ))
    }
    skip <- cer_detect_header_row(file, sheet = sheet)
    df <- as.data.frame(
      readxl::read_excel(file, sheet = sheet, skip = skip,
                         .name_repair = "minimal"),
      stringsAsFactors = FALSE
    )
  } else {
    df <- utils::read.csv(file, stringsAsFactors = FALSE,
                          check.names = FALSE,
                          na.strings = c("", "NA", "N/A", "-"),
                          fileEncoding = "UTF-8")
  }
  names(df) <- cer_clean_names(names(df))
  df
}

#' Find the first row in an XLSX sheet that looks like a header
#'
#' Heuristic: reads up to 15 rows without a header, then finds the
#' first row with at least 3 non-empty character columns. Many CER
#' workbooks have a title + 1-5 narrative rows before the data
#' header, and this avoids hardcoding per-file skip values.
#'
#' @noRd
cer_detect_header_row <- function(file, sheet = 1, max_scan = 15L) {
  probe <- tryCatch(
    readxl::read_excel(file, sheet = sheet, col_names = FALSE,
                       n_max = max_scan, .name_repair = "minimal"),
    error = function(e) NULL
  )
  if (is.null(probe) || nrow(probe) == 0L) return(0L)

  is_string_row <- function(row) {
    vals <- unlist(row, use.names = FALSE)
    vals <- vals[!is.na(vals) & nzchar(as.character(vals))]
    # A header row has at least 3 cells, all character-like, and
    # no cell containing newlines (narrative blocks break this).
    if (length(vals) < 3L) return(FALSE)
    chars <- suppressWarnings(as.character(vals))
    has_newline <- any(grepl("\n", chars))
    looks_numeric <- suppressWarnings(!any(is.na(as.numeric(chars))))
    !has_newline && !looks_numeric
  }

  for (i in seq_len(nrow(probe))) {
    if (is_string_row(probe[i, , drop = FALSE])) {
      return(i - 1L)  # `skip` counts rows before the header.
    }
  }
  0L
}

#' Case-insensitive substring filter on a character column
#'
#' @param df Data frame.
#' @param col Column name (character scalar).
#' @param patterns Character vector of patterns to OR together.
#' @return Filtered data frame.
#' @noRd
cer_filter_like <- function(df, col, patterns) {
  if (is.null(patterns) || length(patterns) == 0L) return(df)
  if (!col %in% names(df)) return(df)
  pattern <- paste(tolower(patterns), collapse = "|")
  keep <- grepl(pattern, tolower(df[[col]]), fixed = FALSE)
  df[keep, , drop = FALSE]
}

#' Exact-match filter on a character column, case-insensitive
#' @noRd
cer_filter_in <- function(df, col, values) {
  if (is.null(values) || length(values) == 0L) return(df)
  if (!col %in% names(df)) return(df)
  values <- toupper(values)
  keep <- toupper(df[[col]]) %in% values
  df[keep, , drop = FALSE]
}

test_that("cer_accu_methods returns a cer_tbl with the expected columns", {
  m <- cer_accu_methods()
  expect_s3_class(m, "cer_tbl")
  expect_true(all(c("method", "sector") %in% names(m)))
  expect_gt(nrow(m), 0L)
})

test_that("cer_filter_like is case-insensitive and OR-matches", {
  df <- data.frame(method = c("HIR savanna fire", "Landfill gas", "Piggery waste"),
                   stringsAsFactors = FALSE)
  got <- cer:::cer_filter_like(df, "method", c("savanna", "piggery"))
  expect_equal(nrow(got), 2L)
})

test_that("cer_filter_in accepts a vector and is case-insensitive", {
  df <- data.frame(state = c("NSW", "VIC", "QLD"), stringsAsFactors = FALSE)
  got <- cer:::cer_filter_in(df, "state", c("nsw", "qld"))
  expect_equal(sort(got$state), c("NSW", "QLD"))
})

test_that("cer_clean_names snakecases punctuation-heavy headers", {
  got <- cer:::cer_clean_names(c("Project ID", "ACCUs Issued (to date)",
                                  "Method-Name", "  trailing "))
  expect_equal(got, c("project_id", "accus_issued_to_date",
                      "method_name", "trailing"))
})

test_that("cer_accu_projects signature validates status choices", {
  expect_error(cer_accu_projects(status = "invalid"))
})

test_that("cer_accu_projects hits the network when online", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("CER_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")

  p <- cer_accu_projects()
  expect_s3_class(p, "cer_tbl")
  expect_gt(nrow(p), 100L)
})

test_that("cer_accu_contracts hits the network when online", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("CER_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")

  c <- cer_accu_contracts()
  expect_s3_class(c, "cer_tbl")
  expect_gt(nrow(c), 10L)
})

# ACCU Scheme : project register, issuances, contracts, methods, relinquishments

#' @noRd
ACCU_PROJECT_CSV <- "https://cer.gov.au/document/accu-scheme-project-register-0"

#' @noRd
ACCU_CONTRACT_CSV <- "https://cer.gov.au/document/carbon-abatement-contract-register-0"

#' ACCU Scheme project register
#'
#' Download and parse the Clean Energy Regulator's Australian
#' Carbon Credit Unit ('ACCU') Scheme project register. Returns one
#' row per project with proponent, method, state, crediting period,
#' permanence period, and cumulative ACCUs issued and relinquished.
#'
#' The register is updated monthly by the CER, usually around the
#' third week. Downloaded once per R session (or invalidate with
#' [cer_clear_cache()]).
#'
#' @param state Optional character vector of Australian state
#'   codes (e.g. `"NSW"`, `c("VIC", "QLD")`) to filter on. `NULL`
#'   returns projects across all states.
#' @param method Optional character vector of method name
#'   substrings (case-insensitive, e.g. `"savanna"`, `"landfill"`,
#'   `"vegetation"`).
#' @param status Character. One of `"active"` (default), `"revoked"`,
#'   `"completed"`, or `"all"`.
#'
#' @return A `cer_tbl` (data frame) with one row per ACCU project.
#'
#' @source Clean Energy Regulator, ACCU project and contract register:
#'   <https://cer.gov.au/markets/reports-and-data/accu-project-and-contract-register>.
#'   Licensed under CC BY 4.0.
#'
#' @references
#' Commonwealth of Australia. \emph{Carbon Credits (Carbon Farming
#'   Initiative) Act 2011}. Enabling legislation for the ACCU Scheme.
#'
#' Chubb, I. (2022). \emph{Independent Review of Australian Carbon
#'   Credit Units}. Commonwealth of Australia, Department of Climate
#'   Change, Energy, the Environment and Water.
#'
#' Climate Change Authority (2024). \emph{2026 Review of the ACCU
#'   Scheme: Issues paper}.
#'
#' Macintosh, A., Butler, D., Larraondo, P., Evans, M.C., Ansell, D.,
#'   Gibbons, P., Lindenmayer, D., Waschka, M. and Fisher, R.
#'   (2022). "The emperor's new clothes: assessing the integrity
#'   of ACCUs." Australian National University working paper series.
#'
#' @family accu
#' @export
#' @examples
#' \donttest{
#' op <- options(cer.cache_dir = tempdir())
#' p <- cer_accu_projects(state = "NSW")
#' head(p)
#' options(op)
#' }
cer_accu_projects <- function(state = NULL, method = NULL,
                               status = c("active", "revoked", "completed", "all")) {
  status <- match.arg(status)
  df <- cer_fetch_csv(ACCU_PROJECT_CSV)

  df <- cer_filter_in(df, "state", state)
  df <- cer_filter_like(df, "method", method)

  if (status != "all") {
    status_col <- intersect(c("project_status", "status"), names(df))[1]
    if (!is.na(status_col)) {
      df <- df[tolower(df[[status_col]]) == status, , drop = FALSE]
    }
  }

  # Attach post-Chubb integrity flag (under_review, suspended,
  # superseded, expired, or active).
  df <- cer_attach_chubb_flag(df)

  # Warn when the user queried a suspended or under-review method.
  if (!is.null(method)) {
    flagged <- unique(df$chubb_flag[df$chubb_flag %in%
                                     c("suspended", "under_review")])
    if (length(flagged) > 0L) {
      cli::cli_alert_warning(c(
        "Method filter matches {.val {flagged}} methods.",
        "i" = "Chubb (2022) flagged these; do not aggregate without disclosing the integrity tier.",
        "i" = "See {.code cer_method_integrity()} for details."
      ))
    }
  }

  rownames(df) <- NULL
  new_cer_tbl(df,
              source = ACCU_PROJECT_CSV,
              licence = "CC BY 4.0",
              title = "ACCU Scheme project register")
}

#' ACCU issuances by project
#'
#' Derived from the ACCU Scheme project register: returns one row
#' per project with `accus_issued`, project metadata, and total
#' issuance net of relinquishments. For method- or period-level
#' issuance tracking, query [cer_accu_projects()] directly.
#'
#' @param method Optional character vector of method substrings.
#' @param project_id Optional character vector of ERF / EOP project
#'   identifiers.
#' @param from,to Optional `Date` (or YYYY-MM-DD string) for
#'   crediting-period overlap filter. `NULL` returns all projects.
#'
#' @return A `cer_tbl` with one row per project.
#' @source Clean Energy Regulator ACCU project register (CC BY 4.0).
#' @family accu
#' @export
#' @examples
#' \donttest{
#' op <- options(cer.cache_dir = tempdir())
#' i <- cer_accu_issuances(method = "savanna")
#' head(i)
#' options(op)
#' }
cer_accu_issuances <- function(method = NULL, project_id = NULL,
                                from = NULL, to = NULL) {
  df <- cer_fetch_csv(ACCU_PROJECT_CSV)
  df <- cer_filter_like(df, "method", method)
  df <- cer_filter_in(df, "project_id", project_id)

  if (!is.null(from) || !is.null(to)) {
    start_col <- intersect(c("crediting_period_start", "start_date"), names(df))[1]
    end_col <- intersect(c("crediting_period_end", "end_date"), names(df))[1]
    if (!is.na(start_col) && !is.na(end_col)) {
      s <- suppressWarnings(as.Date(df[[start_col]]))
      e <- suppressWarnings(as.Date(df[[end_col]]))
      if (!is.null(from)) df <- df[is.na(e) | e >= as.Date(from), , drop = FALSE]
      if (!is.null(to))   df <- df[is.na(s) | s <= as.Date(to), , drop = FALSE]
    }
  }

  rownames(df) <- NULL
  new_cer_tbl(df,
              source = ACCU_PROJECT_CSV,
              licence = "CC BY 4.0",
              title = "ACCU issuances by project")
}

#' ACCU Carbon Abatement Contract register
#'
#' Government purchase contracts under Emissions Reduction Fund
#' auctions administered by the CER.
#'
#' @param auction Optional character vector of auction identifiers
#'   (e.g. `"ERF Auction 18"`).
#'
#' @return A `cer_tbl` with one row per contract.
#' @source Clean Energy Regulator Carbon Abatement Contract register
#'   (CC BY 4.0).
#' @family accu
#' @export
#' @examples
#' \donttest{
#' op <- options(cer.cache_dir = tempdir())
#' cer_accu_contracts()
#' options(op)
#' }
cer_accu_contracts <- function(auction = NULL) {
  df <- cer_fetch_csv(ACCU_CONTRACT_CSV)
  df <- cer_filter_like(df, "auction", auction)
  rownames(df) <- NULL
  new_cer_tbl(df,
              source = ACCU_CONTRACT_CSV,
              licence = "CC BY 4.0",
              title = "Carbon Abatement Contract register")
}

#' Approved ACCU Scheme methods
#'
#' A static table of ACCU Scheme methods grouped by sector
#' (Agriculture, Energy Efficiency, Landfill and Waste,
#' Mining/Oil/Gas, Vegetation, Savanna, Blue Carbon,
#' Transport). Built from the CER methods index at build time.
#'
#' Because the CER publishes methods as per-method PDFs on the
#' Federal Register of Legislation rather than a machine-readable
#' index, this function returns a package-embedded lookup. The
#' `status` column captures post-Chubb Review changes: methods
#' are flagged as `"active"`, `"expired"`, `"suspended"`, or
#' `"under_review"`. Run [cer_accu_projects()] then
#' `unique(x$method)` to cross-check against currently-registered
#' projects.
#'
#' @return A `cer_tbl` with columns `method`, `sector`,
#'   `short_name`, `status`, `commenced`, and `source_url`.
#' @source Clean Energy Regulator ACCU Scheme methods index
#'   <https://cer.gov.au/schemes/australian-carbon-credit-unit-scheme/accu-scheme-methods>.
#' @family accu
#' @export
#' @examples
#' m <- cer_accu_methods()
#' head(m)
cer_accu_methods <- function() {
  # Embedded via sysdata.rda : regenerated by data-raw/build_accu_methods.R
  df <- get0("accu_methods", envir = asNamespace("cer"))
  if (is.null(df)) {
    # Fallback seed table so the function always works before
    # sysdata has been built. Reflects the post-Chubb method
    # landscape as of April 2026.
    df <- data.frame(
      method = c(
        # Vegetation
        "Human-induced regeneration of a permanent even-aged native forest",
        "Avoided deforestation",
        "Avoided clearing of native regrowth",
        "Native forest from managed regrowth",
        "Reforestation by environmental or mallee plantings (2014)",
        "Reforestation by environmental or mallee plantings (FullCAM 2024)",
        # Savanna
        "Savanna fire management 2018 (emissions avoidance)",
        "Savanna fire management 2018 (sequestration and emissions avoidance)",
        "Savanna fire management 2026",
        # Agriculture
        "Beef cattle herd management",
        "Piggery waste",
        "Measurement of soil carbon sequestration in agricultural systems",
        # Landfill and Waste
        "Landfill gas",
        "Alternative waste treatment",
        # Mining / Oil / Gas
        "Coal mine waste gas",
        "Oil and gas fugitive emissions",
        # Energy Efficiency
        "Energy efficiency in commercial buildings",
        "Industrial equipment upgrades",
        # Blue carbon
        "Tidal restoration of blue carbon ecosystems"
      ),
      sector = c(
        rep("Vegetation", 6L),
        rep("Savanna", 3L),
        rep("Agriculture", 3L),
        rep("Landfill and Waste", 2L),
        rep("Mining/Oil/Gas", 2L),
        rep("Energy Efficiency", 2L),
        "Blue carbon"
      ),
      short_name = c(
        "HIR", "AD", "AC native regrowth", "MNFR",
        "Plantings 2014", "Plantings 2024",
        "Savanna EA 2018", "Savanna EA+S 2018", "Savanna 2026",
        "Beef cattle", "Piggery", "Soil measured",
        "Landfill", "AWT",
        "CMWG", "Oil and gas FE",
        "Commercial", "Industrial",
        "Blue carbon"
      ),
      # Status reflects post-Chubb Review changes (January 2023):
      #  - "under_review": HIR pending IFLM replacement method
      #  - "suspended":    Avoided Deforestation closed to new projects
      #  - "expired":      Plantings 2014 expired 30 September 2024,
      #                    superseded by Plantings 2024 FullCAM method
      #  - "superseded":   Savanna 2018 methods superseded by
      #                    Savanna Fire Management 2026
      #  - "active":       everything else
      status = c(
        "under_review", "suspended", "active", "active",
        "expired", "active",
        "superseded", "superseded", "active",
        "active", "active", "active",
        "active", "active",
        "active", "active",
        "active", "active",
        "active"
      ),
      commenced = as.Date(c(
        "2013-07-01", "2015-04-01", "2015-04-01", "2015-07-01",
        "2014-05-30", "2024-10-01",
        "2018-05-01", "2018-05-01", "2026-01-01",
        "2015-08-12", "2012-11-09", "2018-02-09",
        "2012-08-24", "2015-07-01",
        "2018-03-28", "2022-06-01",
        "2013-07-01", "2015-07-01",
        "2022-06-01"
      )),
      source_url = rep(
        "https://cer.gov.au/schemes/australian-carbon-credit-unit-scheme/accu-scheme-methods",
        19L
      ),
      stringsAsFactors = FALSE
    )
  }
  new_cer_tbl(df,
              source = "https://cer.gov.au/schemes/australian-carbon-credit-unit-scheme/accu-scheme-methods",
              licence = "CC BY 4.0",
              title = "ACCU Scheme methods (post-Chubb)")
}

#' ACCU relinquishments by project
#'
#' Returns projects with non-zero ACCU relinquishments. Useful for
#' tracking voluntary cancellations and compliance obligations.
#'
#' @param project_id Optional character vector of project IDs.
#'
#' @return A `cer_tbl` with one row per project with
#'   `accus_relinquished > 0`.
#' @source Clean Energy Regulator ACCU project register (CC BY 4.0).
#' @family accu
#' @export
#' @examples
#' \donttest{
#' op <- options(cer.cache_dir = tempdir())
#' cer_accu_relinquishments()
#' options(op)
#' }
cer_accu_relinquishments <- function(project_id = NULL) {
  df <- cer_fetch_csv(ACCU_PROJECT_CSV)
  df <- cer_filter_in(df, "project_id", project_id)

  rel_col <- intersect(
    c("accus_relinquished", "relinquished_accus", "relinquishments"),
    names(df)
  )[1]
  if (!is.na(rel_col)) {
    vals <- suppressWarnings(as.numeric(df[[rel_col]]))
    df <- df[!is.na(vals) & vals > 0, , drop = FALSE]
  }
  rownames(df) <- NULL
  new_cer_tbl(df,
              source = ACCU_PROJECT_CSV,
              licence = "CC BY 4.0",
              title = "ACCU relinquishments by project")
}

# QCMR reconciliation: diff an aggregate against the authoritative
# Quarterly Carbon Market Report headline figures.
#
# The CER publishes the QCMR each quarter with agreed totals
# (cumulative ACCU issuances, surrenders, active projects, LGC and
# STC volumes, Safeguard covered emissions, SMC issuances, SRES
# installations). Because ACCU project registers are retroactively
# revised, a live-fetched aggregate today may differ from the
# published QCMR figure for the same quarter. This is the single
# highest-value check in a carbon market research pipeline.

#' QCMR headline reference totals
#'
#' Bundled quarterly Clean Energy Regulator QCMR headline figures
#' for cross-validation of live-fetched aggregates.
#'
#' @param quarter Optional character filter (e.g. `"2024-Q4"`).
#' @param measure Optional character filter (e.g.
#'   `"accu_cumulative_issuances"`).
#'
#' @return A `cer_tbl` of reference totals.
#' @family reconciliation
#' @export
#' @examples
#' cer_qcmr_reference()
#' cer_qcmr_reference(quarter = "2024-Q4")
cer_qcmr_reference <- function(quarter = NULL, measure = NULL) {
  path <- system.file("extdata", "qcmr_headline_totals.csv",
                       package = "cer")
  df <- utils::read.csv(path, stringsAsFactors = FALSE,
                         na.strings = c("", "NA"), check.names = FALSE)
  if (!is.null(quarter)) {
    df <- df[df$quarter %in% quarter, , drop = FALSE]
  }
  if (!is.null(measure)) {
    df <- df[df$measure %in% measure, , drop = FALSE]
  }
  rownames(df) <- NULL
  new_cer_tbl(df,
              source = "https://cer.gov.au/markets/reports-and-data/quarterly-carbon-market-reports",
              licence = "CC BY 4.0",
              title = "QCMR headline reference totals")
}

#' Reconcile an aggregate against the QCMR headline figure
#'
#' Compares a scalar or data-frame sum against the CER Quarterly
#' Carbon Market Report headline for the same quarter and measure.
#' Warns on absolute percentage gap > 2% (default). Small gaps
#' (sub-1%) are expected because ACCU project registers are
#' retroactively revised between QCMR releases.
#'
#' @param value Numeric scalar or data frame. If a data frame, pass
#'   `sum_column` to pick which numeric column to sum.
#' @param quarter Character, e.g. `"2024-Q4"` or `"2023-24"`.
#' @param measure Character, matching a `measure` value in
#'   [cer_qcmr_reference()]. For example
#'   `"accu_cumulative_issuances"`,
#'   `"safeguard_covered_emissions_mt"`, `"smc_issuances"`.
#' @param sum_column Column name to sum when `value` is a data frame.
#' @param warn_pct Warn if |pct_diff| > this threshold. Default 0.02.
#'
#' @return A one-row data frame: `measure`, `quarter`, `value`,
#'   `reference`, `diff`, `pct_diff`, `unit`, `source`.
#'
#' @references
#' Clean Energy Regulator (quarterly). \emph{Quarterly Carbon Market
#'   Report}. \url{https://cer.gov.au/markets/reports-and-data/quarterly-carbon-market-reports}
#'
#' @family reconciliation
#' @export
#' @examples
#' cer_reconcile(value = 185e6,
#'               quarter = "2024-Q4",
#'               measure = "accu_cumulative_issuances")
cer_reconcile <- function(value, quarter, measure,
                           sum_column = NULL,
                           warn_pct = 0.02) {
  if (is.data.frame(value)) {
    cols <- names(value)[vapply(value, is.numeric, logical(1))]
    if (is.null(sum_column)) {
      if (length(cols) == 0L) {
        cli::cli_abort("No numeric columns in {.arg value}.")
      }
      if (length(cols) > 1L) {
        cli::cli_abort(c(
          "Multiple numeric columns in {.arg value}.",
          "i" = "Pass {.arg sum_column}: {.val {cols}}."
        ))
      }
      sum_column <- cols
    }
    v <- sum(value[[sum_column]], na.rm = TRUE)
  } else {
    stopifnot(is.numeric(value), length(value) == 1L)
    v <- value
  }

  ref <- as.data.frame(cer_qcmr_reference())
  hit <- ref[ref$quarter == quarter & ref$measure == measure, ]
  if (nrow(hit) == 0L) {
    cli::cli_abort(c(
      "No QCMR reference for {.val {measure}} in {.val {quarter}}.",
      "i" = "Available measures: {.val {unique(ref$measure)}}.",
      "i" = "Available quarters: {.val {unique(ref$quarter)}}."
    ))
  }
  ref_v <- hit$value[1L]
  diff <- v - ref_v
  pct <- diff / ref_v

  if (abs(pct) > warn_pct) {
    cli::cli_warn(c(
      "Reconciliation gap {round(100 * pct, 2)}% for {.val {measure}} ({.val {quarter}}).",
      "i" = "Live: {.val {format(v, big.mark = ',', scientific = FALSE)}}",
      "i" = "QCMR reference: {.val {format(ref_v, big.mark = ',', scientific = FALSE)}}",
      "i" = "CER data is retroactively revised; check for recent relinquishments or corrections."
    ))
  }

  data.frame(
    measure   = measure,
    quarter   = quarter,
    value     = v,
    reference = ref_v,
    diff      = diff,
    pct_diff  = pct,
    unit      = hit$unit[1L],
    source    = hit$source[1L],
    stringsAsFactors = FALSE
  )
}

test_that("cer_qcmr_reference returns rows", {
  df <- cer_qcmr_reference()
  expect_s3_class(df, "data.frame")
  expect_gt(nrow(df), 10L)
})

test_that("cer_qcmr_reference filters by quarter", {
  df <- cer_qcmr_reference(quarter = "2024-Q4")
  expect_true(all(df$quarter == "2024-Q4"))
})

# Level 2: reconcile at exact match returns diff = 0.
test_that("cer_reconcile returns zero diff on exact match", {
  res <- cer_reconcile(value = 185000000,
                       quarter = "2024-Q4",
                       measure = "accu_cumulative_issuances")
  expect_equal(res$diff, 0)
  expect_equal(res$pct_diff, 0)
})

# Level 3: warning fires when gap > 2%.
test_that("cer_reconcile warns on gap > 2%", {
  expect_warning(
    cer_reconcile(value = 200000000,
                   quarter = "2024-Q4",
                   measure = "accu_cumulative_issuances"),
    "Reconciliation gap"
  )
})

# Level 3: pct_diff sign is consistent with (value - reference).
test_that("pct_diff sign is consistent with diff", {
  high <- suppressWarnings(cer_reconcile(200000000, "2024-Q4",
                                           "accu_cumulative_issuances"))
  low  <- suppressWarnings(cer_reconcile(170000000, "2024-Q4",
                                           "accu_cumulative_issuances"))
  expect_gt(high$diff, 0)
  expect_gt(high$pct_diff, 0)
  expect_lt(low$diff, 0)
  expect_lt(low$pct_diff, 0)
})

test_that("cer_reconcile errors on unknown measure/quarter", {
  expect_error(
    cer_reconcile(1e6, "2999-Q1", "not_a_measure"),
    "No QCMR reference"
  )
})

# Reproducibility: session-level snapshot date pin.
#
# Carbon market data is retroactively revised (ACCU relinquishments,
# project transfers, method reclassifications, Safeguard amendments).
# A snapshot_date pin lets the user record their intended vintage
# and pair it with SHA-256 integrity for a reproducible audit trail.

#' Pin or inspect the session snapshot date
#'
#' Call once at the top of an analysis script to declare the vintage
#' of CER data you intend to use. Every subsequent `cer_*` fetch
#' records this date in the `cer_tbl` provenance header, in
#' [cer_manifest()] entries, and in [cer_cite()] output.
#'
#' This matters more for CER data than for most government data:
#' projects relinquish ACCUs retroactively, the CER transfers
#' projects between proponents, methods are reclassified (e.g.
#' post-Chubb-Review status changes), and the Quarterly Carbon
#' Market Report occasionally restates prior quarters. A snapshot
#' pin is the minimum evidence a reviewer needs to verify a
#' published carbon-market claim.
#'
#' @param date ISO `"YYYY-MM-DD"` character, `Date`, or `POSIXct`.
#'   Pass `NULL` to clear.
#'
#' @return Invisibly, the new pinned date (as `Date`), or `NULL`.
#' @family reproducibility
#' @export
#' @examples
#' cer_snapshot("2026-04-24")
#' cer_snapshot()
#' cer_snapshot(NULL)
cer_snapshot <- function(date) {
  if (missing(date)) {
    return(.cer_env$snapshot_date %||% NULL)
  }
  if (is.null(date)) {
    .cer_env$snapshot_date <- NULL
    return(invisible(NULL))
  }
  d <- tryCatch(as.Date(date), error = function(e) NA)
  if (is.na(d)) {
    cli::cli_abort(
      "Could not parse {.arg date} as a date. Use {.val YYYY-MM-DD}."
    )
  }
  .cer_env$snapshot_date <- d
  invisible(d)
}

#' @noRd
cer_snapshot_str <- function() {
  d <- .cer_env$snapshot_date %||% NULL
  if (is.null(d)) NA_character_ else format(d, "%Y-%m-%d")
}

test_that("cer_snapshot gets and sets the pin", {
  on.exit(cer_snapshot(NULL), add = TRUE)
  cer_snapshot(NULL)
  expect_null(cer_snapshot())

  d <- cer_snapshot("2026-04-24")
  expect_s3_class(d, "Date")
  expect_equal(format(cer_snapshot(), "%Y-%m-%d"), "2026-04-24")
})

test_that("cer_snapshot clears on NULL", {
  cer_snapshot("2026-04-24")
  cer_snapshot(NULL)
  expect_null(cer_snapshot())
})

test_that("cer_snapshot errors on bad input", {
  on.exit(cer_snapshot(NULL), add = TRUE)
  expect_error(cer_snapshot("not a date"), "parse")
})

# Federal Register of Legislation URL resolver.
#
# Every ACCU method and NGER/Safeguard rule is a Legislative
# Instrument on the Federal Register of Legislation
# (legislation.gov.au). The "F<YYYY>L<NNNNN>" identifier is stable
# and authoritative; CER webpage slugs are not.

#' Build a Federal Register of Legislation URL
#'
#' Given a Legislative Instrument identifier (e.g. `"F2024L01292"`
#' for the Environmental Plantings 2024 method), return the stable
#' URL on the Federal Register of Legislation. This is the
#' authoritative source of truth for ACCU methods, Safeguard Rule
#' amendments, and NGER Determinations.
#'
#' @param instrument Character identifier (e.g. `"F2024L01292"`).
#'   Case-insensitive.
#'
#' @return A length-1 character URL string.
#'
#' @references
#' Commonwealth of Australia. \emph{Legislation Act 2003} establishes
#'   the Federal Register of Legislation as the authoritative
#'   source for Legislative Instruments.
#'
#' @family reproducibility
#' @export
#' @examples
#' cer_legislative_instrument("F2024L01292")
#' cer_legislative_instrument("F2015L00398")
cer_legislative_instrument <- function(instrument) {
  stopifnot(is.character(instrument), length(instrument) == 1L)
  instrument <- toupper(trimws(instrument))
  if (!grepl("^F[0-9]{4}[LC][0-9]+$", instrument)) {
    cli::cli_warn(c(
      "{.val {instrument}} does not match the Legislative Instrument pattern.",
      "i" = "Expected format like {.val F2024L01292} (F + YYYY + L or C + digits)."
    ))
  }
  sprintf("https://www.legislation.gov.au/Details/%s", instrument)
}

test_that("cer_legislative_instrument builds a URL", {
  u <- cer_legislative_instrument("F2024L01292")
  expect_match(u, "legislation\\.gov\\.au/Details/F2024L01292")
})

test_that("cer_legislative_instrument warns on bad identifier", {
  expect_warning(cer_legislative_instrument("garbage"),
                 "does not match")
})

test_that("cer_legislative_instrument uppercases input", {
  u <- cer_legislative_instrument("f2024l01292")
  expect_match(u, "F2024L01292")
})

# HTTP helpers : httr2 wrapper with local caching

#' @noRd
CER_BASE_URL <- "https://cer.gov.au"

#' @noRd
cer_user_agent <- function() {
  paste0("cer R package/", utils::packageVersion("cer"),
         " (+https://github.com/charlescoverdale/cer)")
}

#' @noRd
cer_request <- function(url, timeout = 120) {
  httr2::request(url) |>
    httr2::req_user_agent(cer_user_agent()) |>
    httr2::req_throttle(rate = 5 / 10) |>
    httr2::req_timeout(timeout) |>
    httr2::req_retry(max_tries = 3) |>
    httr2::req_error(is_error = function(r) FALSE)
}

#' Download a file to the cache and return the local path
#'
#' @param url URL to download.
#' @param cache Logical. Use an existing cached file if present.
#' @return Path to a local file.
#' @noRd
cer_download_cached <- function(url, cache = TRUE) {
  d <- cer_cache_dir()
  ext <- tools::file_ext(url)
  ext <- if (nzchar(ext)) paste0(".", ext) else ""
  file <- file.path(d, paste0(cer_digest_url(url), ext))

  if (cache && file.exists(file)) {
    cer_sha_verify(file)
    .cer_manifest_append(url = url, file = file)
    return(file)
  }

  cli::cli_progress_step("Downloading {.url {url}}")

  resp <- tryCatch(
    cer_request(url) |>
      httr2::req_perform(path = file),
    error = function(e) {
      if (file.exists(file)) unlink(file)
      cli::cli_abort(c(
        "Download failed.",
        "x" = conditionMessage(e)
      ))
    }
  )

  if (!is.null(resp) && httr2::resp_status(resp) >= 400L) {
    unlink(file, force = TRUE)
    cli::cli_abort(c(
      "HTTP {httr2::resp_status(resp)} from {.url {url}}.",
      "i" = "The CER may have moved or renamed the resource."
    ))
  }

  cer_sha_write(file)
  .cer_manifest_append(url = url, file = file)
  file
}

#' Scrape document links from a CER landing page matching a pattern
#'
#' Several CER data pages host the XLSX under a slug that shifts
#' year-to-year or quarter-to-quarter (e.g. QCMR, Safeguard,
#' NGER). Rather than hard-code each slug, this helper fetches
#' the landing page HTML and returns all `/document/*` links whose
#' slug matches a regex.
#'
#' @param page_url Landing page URL.
#' @param pattern Regex to filter the link slug.
#' @return Character vector of fully-qualified document URLs
#'   (absolute). Empty if nothing matches.
#' @noRd
cer_scrape_links <- function(page_url, pattern) {
  html <- tryCatch(
    cer_request(page_url) |>
      httr2::req_perform() |>
      httr2::resp_body_string(),
    error = function(e) NA_character_
  )
  if (is.na(html)) return(character(0L))

  hrefs <- regmatches(html, gregexpr('href="(/document/[^"]+)"', html))[[1]]
  hrefs <- gsub('^href="|"$', "", hrefs)
  hrefs <- unique(hrefs)
  keep <- grepl(pattern, hrefs, ignore.case = TRUE)
  paste0(CER_BASE_URL, hrefs[keep])
}

#' Stable URL hash for cache filenames
#'
#' A dependency-free weighted-checksum hash. Not cryptographic:
#' just collision-resistant enough for a single-user local cache.
#'
#' @param url URL to hash.
#' @return A 10-digit numeric string plus URL-length suffix.
#' @noRd
cer_digest_url <- function(url) {
  chars <- utf8ToInt(url)
  weights <- seq_along(chars)
  checksum <- sum(as.numeric(chars) * weights) %% (2^31 - 1)
  sprintf("%010.0f_%04d", as.numeric(checksum), nchar(url) %% 10000L)
}

test_that("cer_digest_url is deterministic and length-varying", {
  d1 <- cer:::cer_digest_url("https://cer.gov.au/a")
  d2 <- cer:::cer_digest_url("https://cer.gov.au/a")
  d3 <- cer:::cer_digest_url("https://cer.gov.au/b")

  expect_equal(d1, d2)
  expect_false(identical(d1, d3))
  expect_match(d1, "^[0-9]+_[0-9]+$")
})

test_that("cer_user_agent includes the package version", {
  ua <- cer:::cer_user_agent()
  expect_match(ua, "^cer R package/")
  expect_match(ua, "github.com/charlescoverdale/cer")
})

test_that("cer_request returns a httr2 request object", {
  req <- cer:::cer_request("https://cer.gov.au")
  expect_s3_class(req, "httr2_request")
})

test_that("cer_download_cached uses existing cached file when present", {
  tmp <- tempfile("cer_cache_")
  op <- options(cer.cache_dir = tmp)
  on.exit(options(op), add = TRUE)
  dir.create(tmp, recursive = TRUE)

  url <- "https://cer.gov.au/test-file.csv"
  hash <- cer:::cer_digest_url(url)
  fake_path <- file.path(tmp, paste0(hash, ".csv"))
  writeLines("col1,col2\n1,2", fake_path)

  got <- cer:::cer_download_cached(url, cache = TRUE)
  expect_equal(normalizePath(got), normalizePath(fake_path))
})

# SHA-256 integrity: every cached file has a sidecar `.sha256`
# recording the hash at first download. Subsequent cache hits
# verify against the sidecar and warn on drift. Critical for
# carbon market data because the CER occasionally restates prior
# releases (e.g. Safeguard facility corrections, ACCU retroactive
# relinquishments).

#' Compute the SHA-256 digest of a file
#'
#' @param file Path to a local file.
#' @return A length-1 character string (hex digest), or `NA` if
#'   the file does not exist. Uses `digest::digest()` when the
#'   `digest` package is available (recommended for research work),
#'   falling back to `tools::md5sum()` otherwise.
#' @family reproducibility
#' @export
#' @examples
#' f <- tempfile()
#' writeLines("hello", f)
#' cer_sha256(f)
cer_sha256 <- function(file) {
  if (!file.exists(file)) return(NA_character_)
  if (requireNamespace("digest", quietly = TRUE)) {
    return(digest::digest(file = file, algo = "sha256"))
  }
  unname(tools::md5sum(file))
}

#' @noRd
cer_sha_sidecar <- function(file) {
  paste0(file, ".sha256")
}

#' @noRd
cer_sha_write <- function(file) {
  sha <- cer_sha256(file)
  if (is.na(sha)) return(invisible(NA_character_))
  writeLines(sha, cer_sha_sidecar(file))
  invisible(sha)
}

#' @noRd
cer_sha_read <- function(file) {
  side <- cer_sha_sidecar(file)
  if (!file.exists(side)) return(NA_character_)
  tryCatch(readLines(side, n = 1L, warn = FALSE),
           error = function(e) NA_character_)
}

#' @noRd
cer_sha_verify <- function(file) {
  expected <- cer_sha_read(file)
  if (is.na(expected) || !nzchar(expected)) {
    cer_sha_write(file)
    return(invisible(TRUE))
  }
  actual <- cer_sha256(file)
  if (!identical(expected, actual)) {
    cli::cli_warn(c(
      "SHA-256 mismatch for cached file {.path {basename(file)}}.",
      "i" = "The CER may have restated this data release.",
      "i" = "Expected {.val {substr(expected, 1, 12)}...}",
      "i" = "Got      {.val {substr(actual, 1, 12)}...}",
      "i" = "Run {.code cer_clear_cache()} and re-fetch to resolve."
    ))
    return(invisible(FALSE))
  }
  invisible(TRUE)
}

test_that("cer_sha256 returns a hex string for a real file", {
  f <- tempfile()
  writeLines("hello", f)
  sha <- cer_sha256(f)
  expect_type(sha, "character")
  expect_true(nzchar(sha))
})

test_that("cer_sha256 returns NA for missing file", {
  expect_true(is.na(cer_sha256(tempfile())))
})

# Level 2: NIST FIPS 180-4 empty-string vector.
test_that("cer_sha256 matches NIST FIPS 180-4 empty-string vector", {
  skip_if_not_installed("digest")
  f <- tempfile()
  file.create(f)
  expect_equal(
    cer_sha256(f),
    "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855"
  )
})

# Level 4: cross-implementation against openssl.
test_that("cer_sha256 agrees with openssl::sha256", {
  skip_if_not_installed("openssl")
  skip_if_not_installed("digest")
  f <- tempfile()
  writeLines("reproducibility audit", f)
  ours   <- cer_sha256(f)
  theirs <- paste(as.character(openssl::sha256(file(f))), collapse = "")
  expect_equal(tolower(ours), tolower(theirs))
})

# Zenodo deposit helper. Mints a DOI for a CER data vintage so a
# paper can cite the exact snapshot rather than a URL that may
# return different bytes after a retroactive CER restatement.

#' Prepare a Zenodo deposit payload for the session manifest
#'
#' Builds Zenodo-shaped metadata for a data deposit using the
#' current [cer_manifest()] and the snapshot pin set via
#' [cer_snapshot()]. Dry run by default; pass `upload = TRUE` and
#' a token to actually deposit.
#'
#' Because CER data is revised retroactively, Zenodo DOIs are the
#' strongest reproducibility artefact available: they freeze the
#' exact bytes of the ACCU project register, Safeguard facility
#' file, or QCMR workbook at the date of analysis.
#'
#' @param title Deposit title. Defaults to a snapshot-date string.
#' @param description Free-text description. Defaults to a short
#'   auto-generated summary of datasets fetched.
#' @param creators List of creator records.
#' @param keywords Character vector of keywords.
#' @param upload Logical. If `TRUE`, POSTs to Zenodo.
#' @param sandbox Logical. If `TRUE`, uses Zenodo Sandbox for tests.
#' @param token Zenodo personal access token. Defaults to
#'   `Sys.getenv("ZENODO_TOKEN")`.
#'
#' @return A list with `payload`, `manifest_path`, and (on upload)
#'   `deposit_id`, `doi_prereserve`, `url`.
#' @family reproducibility
#' @export
#' @examples
#' \donttest{
#' cer_snapshot("2026-04-24")
#' cer_deposit_zenodo(
#'   title = "CER data snapshot for working paper v1",
#'   creators = list(list(name = "Author, A.")),
#'   upload = FALSE
#' )
#' }
cer_deposit_zenodo <- function(title = NULL,
                                description = NULL,
                                creators = list(list(name = "Anonymous")),
                                keywords = c("Clean Energy Regulator",
                                             "carbon markets", "Australia",
                                             "ACCU", "Safeguard",
                                             "reproducibility"),
                                upload = FALSE,
                                sandbox = FALSE,
                                token = Sys.getenv("ZENODO_TOKEN")) {
  man <- cer_manifest("df")
  if (nrow(man) == 0L) {
    cli::cli_warn(c(
      "Session manifest is empty.",
      "i" = "Fetch some datasets (e.g. {.code cer_accu_projects()}) first."
    ))
  }

  snap <- cer_snapshot_str()
  if (is.null(title) || is.na(title)) {
    title <- sprintf(
      "CER data snapshot %s",
      if (!is.na(snap)) snap else format(Sys.Date(), "%Y-%m-%d")
    )
  }
  if (is.null(description) || is.na(description)) {
    description <- sprintf(
      paste0("Snapshot of %d Clean Energy Regulator dataset(s) ",
             "fetched via the 'cer' R package (v%s). Snapshot ",
             "date: %s. Datasets: %s."),
      nrow(man), utils::packageVersion("cer"),
      if (!is.na(snap)) snap else "unset",
      paste(unique(man$title), collapse = "; ")
    )
  }

  payload <- list(
    metadata = list(
      title = title,
      description = description,
      upload_type = "dataset",
      creators = creators,
      keywords = as.list(keywords),
      license = "cc-by-4.0",
      access_right = "open",
      related_identifiers = lapply(unique(man$url), function(u) {
        list(identifier = u, relation = "isDerivedFrom", scheme = "url")
      })
    )
  )

  manifest_path <- file.path(tempdir(),
                              paste0("cer_manifest_",
                                     format(Sys.time(), "%Y%m%d_%H%M%S"),
                                     ".csv"))
  cer_manifest_write(manifest_path, format = "csv")

  result <- list(payload = payload, manifest_path = manifest_path)

  if (!isTRUE(upload)) {
    cli::cli_inform(c(
      "v" = "Dry run: payload built, manifest staged at {.path {manifest_path}}.",
      "i" = "Call with {.code upload = TRUE} to deposit to Zenodo."
    ))
    return(invisible(result))
  }

  if (!nzchar(token)) {
    cli::cli_abort(c(
      "No Zenodo token.",
      "i" = "Set {.envvar ZENODO_TOKEN} or pass {.arg token}."
    ))
  }

  if (!requireNamespace("jsonlite", quietly = TRUE)) {
    cli::cli_abort("Upload requires the {.pkg jsonlite} package.")
  }

  base <- if (isTRUE(sandbox)) {
    "https://sandbox.zenodo.org/api"
  } else {
    "https://zenodo.org/api"
  }

  create_url <- sprintf("%s/deposit/depositions", base)
  resp <- httr2::request(create_url) |>
    httr2::req_headers(Authorization = paste("Bearer", token)) |>
    httr2::req_body_json(payload) |>
    httr2::req_method("POST") |>
    httr2::req_error(is_error = function(r) FALSE) |>
    httr2::req_perform()

  if (httr2::resp_status(resp) >= 400L) {
    cli::cli_abort(c(
      "Zenodo returned HTTP {httr2::resp_status(resp)}.",
      "x" = httr2::resp_body_string(resp)
    ))
  }

  deposit <- jsonlite::fromJSON(httr2::resp_body_string(resp),
                                 simplifyVector = FALSE)
  upload_url <- deposit$links$bucket %||%
                sprintf("%s/files", deposit$links$self)

  file_resp <- httr2::request(sprintf("%s/%s", upload_url,
                                       basename(manifest_path))) |>
    httr2::req_headers(Authorization = paste("Bearer", token)) |>
    httr2::req_method("PUT") |>
    httr2::req_body_file(manifest_path) |>
    httr2::req_perform()

  if (httr2::resp_status(file_resp) >= 400L) {
    cli::cli_warn(c(
      "Manifest upload returned HTTP {httr2::resp_status(file_resp)}.",
      "i" = "Deposit created but file upload failed. Complete manually."
    ))
  }

  cli::cli_inform(c(
    "v" = "Zenodo deposit created.",
    "i" = "DOI (pre-reserved): {.val {deposit$metadata$prereserve_doi$doi}}",
    "i" = "Review and publish at {.url {deposit$links$html}}"
  ))

  result$deposit_id <- deposit$id
  result$doi_prereserve <- deposit$metadata$prereserve_doi$doi
  result$url <- deposit$links$html
  invisible(result)
}

#' cer: Australian Clean Energy Regulator Data
#'
#' Tidy R access to data published by the Australian Clean Energy
#' Regulator on carbon credits, the Safeguard Mechanism, renewable
#' energy certificates, and greenhouse gas reporting.
#'
#' @section Main function families:
#' - [cer_accu_projects()], [cer_accu_issuances()],
#'   [cer_accu_contracts()], [cer_accu_methods()],
#'   [cer_accu_relinquishments()]: Australian Carbon Credit Unit
#'   ('ACCU') Scheme.
#' - [cer_safeguard_facilities()]: Safeguard Mechanism covered
#'   emissions, baselines, and SMC issuance.
#' - [cer_nger_corporate()], [cer_nger_electricity()]: National
#'   Greenhouse and Energy Reporting ('NGER') scheme.
#' - [cer_lgc_power_stations()]: Large-scale Renewable Energy Target
#'   ('LRET') accredited power stations.
#' - [cer_sres_installations()]: Small-scale Renewable Energy Scheme
#'   ('SRES') postcode installation data.
#' - [cer_qcmr()]: Quarterly Carbon Market Report data workbook.
#' - [cer_cache_info()], [cer_clear_cache()]: cache management.
#'
#' @section Data source:
#' All data is published by the Clean Energy Regulator at
#' <https://cer.gov.au/markets/reports-and-data> under a Creative
#' Commons Attribution 4.0 International licence.
#'
#' @keywords internal
#' @concept Australian climate policy
#' @concept carbon markets
#' @concept ACCU
#' @concept Safeguard Mechanism
#' @concept NGER
"_PACKAGE"

utils::globalVariables(c("accu_methods"))

# Safeguard Mechanism + NGER

#' @noRd
SAFEGUARD_LANDING <- "https://cer.gov.au/markets/reports-and-data/safeguard-data"

#' @noRd
NGER_LANDING <- "https://cer.gov.au/markets/reports-and-data/nger-reporting-data-and-registers"

#' @noRd
safeguard_urls <- function(year) {
  yr_lo <- year - 1L
  yr_hi <- year %% 100L
  yr_label <- sprintf("%d-%02d", yr_lo, yr_hi)

  # CER uses three different URL conventions across years:
  #   2023-24+: /safeguard-data/<year>-baselines-and-emissions-data/
  #             -> /document/baselines-and-emissions-table-<year>
  #   ~2022-23: /safeguard-facility-covered-emissions-data-<year>/
  #             -> /document/safeguard-facilities-data-<year>
  #   Pre-2022: /safeguard-facility-reported-emissions-data/safeguard-facility-reported-emissions-<year>/
  #             -> similar slug inside
  # Try each subpage, collect /document/ links, filter to data-table
  # slugs (not explanation letters, not PDFs).
  subpages <- c(
    sprintf("%s/%s-baselines-and-emissions-data", SAFEGUARD_LANDING, yr_label),
    sprintf("https://cer.gov.au/markets/reports-and-data/safeguard-facility-covered-emissions-data-%s", yr_label),
    sprintf("https://cer.gov.au/markets/reports-and-data/safeguard-facility-reported-emissions-data/safeguard-facility-reported-emissions-%s", yr_label)
  )

  # Slugs that identify the headline data file (exclude per-facility
  # "explanation-letter" PDFs and method determinations).
  table_rx <- paste0(
    "baselines-and-emissions-table-", yr_label, "|",
    "safeguard-facilities-data-", yr_label, "|",
    "safeguard-facility-covered-emissions-", yr_label, "|",
    "safeguard-facility-reported-emissions-", yr_label
  )

  hits <- character(0)
  for (sp in subpages) {
    hits <- cer_scrape_links(sp, table_rx)
    if (length(hits) > 0L) break
  }
  if (length(hits) == 0L) {
    cli::cli_abort(c(
      "No safeguard dataset found for reporting year {yr_lo}-{yr_hi}.",
      "i" = "The CER landing page is {.url {SAFEGUARD_LANDING}}."
    ))
  }
  # Prefer CSV (`-0` or `-csv` suffix) over XLSX (`-excel` or plain).
  csv <- hits[grepl("-0$|-csv$", hits)]
  xlsx <- setdiff(hits, csv)
  list(csv = if (length(csv) > 0L) csv[1L] else NA_character_,
       xlsx = if (length(xlsx) > 0L) xlsx[1L] else NA_character_)
}

#' @noRd
nger_urls <- function(year, kind = c("corporate", "electricity")) {
  kind <- match.arg(kind)
  yr_lo <- year - 1L
  yr_hi <- year %% 100L
  yr_label <- sprintf("%d-%02d", yr_lo, yr_hi)
  pattern <- switch(kind,
    corporate   = sprintf("corporate.*%s|%s.*corporate", yr_label, yr_label),
    electricity = sprintf("(electricity|greenhouse-and-energy-register).*%s|%s.*(electricity|greenhouse)",
                          yr_label, yr_label)
  )
  hits <- cer_scrape_links(NGER_LANDING, pattern)
  if (length(hits) == 0L) {
    cli::cli_abort(c(
      "No NGER {kind} dataset found for reporting year {yr_lo}-{yr_hi}.",
      "i" = "The CER landing page is {.url {NGER_LANDING}}."
    ))
  }
  csv <- hits[grepl("-0$", hits)]
  if (length(csv) > 0L) return(csv[1L])
  hits[1L]
}

#' Safeguard Mechanism facility-level data
#'
#' Covered emissions, baselines, Safeguard Mechanism Credits (SMCs)
#' issued or surrendered, and abatement information for facilities
#' above the 100,000 tonne CO2e threshold.
#'
#' Published annually by the CER on 15 April following each
#' reporting year. The download URL is resolved dynamically from
#' the CER's Safeguard landing page, so no slug changes here are
#' required when new years publish.
#'
#' @param year Integer. Reporting year ending 30 June (e.g. `2025`
#'   for 2024-25). Defaults to the most recently released year.
#'
#' @return A `cer_tbl` with one row per facility.
#'
#' @source Clean Energy Regulator, Safeguard data:
#'   <https://cer.gov.au/markets/reports-and-data/safeguard-data>.
#'   Licensed under CC BY 4.0.
#'
#' @references
#' Commonwealth of Australia. \emph{National Greenhouse and Energy
#'   Reporting Act 2007}.
#'
#' Commonwealth of Australia. \emph{National Greenhouse and Energy
#'   Reporting (Safeguard Mechanism) Rule 2015} (as amended).
#'
#' Commonwealth of Australia (2023). \emph{Safeguard Mechanism
#'   (Crediting) Amendment Act 2023}. Introduced declining industry
#'   baselines (4.9% p.a. nominal) and Safeguard Mechanism Credits
#'   (SMCs).
#'
#' Climate Change Authority (2023). \emph{Review of the Safeguard
#'   Mechanism}.
#'
#' @family safeguard
#' @export
#' @examples
#' \donttest{
#' op <- options(cer.cache_dir = tempdir())
#' try({
#'   f <- cer_safeguard_facilities(year = 2025)
#'   head(f)
#' })
#' options(op)
#' }
cer_safeguard_facilities <- function(year = 2025) {
  stopifnot(is.numeric(year), length(year) == 1L, year >= 2017L, year <= 2030L)
  urls <- safeguard_urls(year)

  # URL suffix convention is inconsistent across years (ACCU inverse
  # of Safeguard), so fall back on file magic bytes for dispatch.
  candidate <- if (!is.na(urls$csv)) urls$csv else urls$xlsx
  df <- cer_fetch_auto(candidate)

  # Regime flag: pre-reform (FY 2016-17 to 2022-23) vs post-reform
  # (FY 2023-24 onward). The reform changed baseline methodology
  # fundamentally; a cross-regime time series should not be
  # summarised as a single trend without this split.
  fy_start <- year - 1L
  df$regime <- cer_safeguard_regime(fy_start)
  df$financial_year <- sprintf("%d-%02d", fy_start, fy_start %% 100L + 1L)

  # TEBA flag: join declared Trade-Exposed Baseline-Adjusted list.
  teba <- as.data.frame(cer_safeguard_teba_facilities())
  fac_col <- intersect(c("facility_name", "facility"), names(df))[1]
  if (!is.na(fac_col)) {
    teba_set <- tolower(teba$facility_name)
    df$teba_declared <- tolower(df[[fac_col]]) %in% teba_set
  } else {
    df$teba_declared <- NA
  }

  rownames(df) <- NULL
  new_cer_tbl(df,
              source = SAFEGUARD_LANDING,
              licence = "CC BY 4.0",
              title = paste0("Safeguard facilities ", year - 1L, "-", year %% 100L))
}

#' NGER corporate emissions and energy data
#'
#' Controlling-corporation-level Scope 1 and Scope 2 emissions and
#' energy use from the National Greenhouse and Energy Reporting
#' ('NGER') scheme.
#'
#' Published annually by the CER on 28 February. Facility-level
#' Scope 1 and 2 data is only published for the electricity
#' sector; non-electricity sectors report at this
#' controlling-corporation level only.
#'
#' @param year Integer. Reporting year ending 30 June.
#'
#' @return A `cer_tbl` with one row per reporting corporation.
#'
#' @source Clean Energy Regulator, NGER reporting data and registers:
#'   <https://cer.gov.au/markets/reports-and-data/nger-reporting-data-and-registers>.
#'   Licensed under CC BY 4.0.
#'
#' @references
#' Commonwealth of Australia. \emph{National Greenhouse and Energy
#'   Reporting Act 2007}; \emph{National Greenhouse and Energy
#'   Reporting (Measurement) Determination 2008} (as amended
#'   annually).
#'
#' Greenhouse Gas Protocol (2004). \emph{A Corporate Accounting
#'   and Reporting Standard (Revised Edition)}. World Resources
#'   Institute.
#'
#' Department of Climate Change, Energy, the Environment and Water
#'   (annual). \emph{NGER Technical Guidelines}. Methodology for
#'   Scope 1 and Scope 2 calculations.
#'
#' @family nger
#' @export
#' @examples
#' \donttest{
#' op <- options(cer.cache_dir = tempdir())
#' try({
#'   x <- cer_nger_corporate(year = 2025)
#'   head(x)
#' })
#' options(op)
#' }
cer_nger_corporate <- function(year = 2025) {
  stopifnot(is.numeric(year), length(year) == 1L, year >= 2009L, year <= 2030L)
  url <- nger_urls(year, "corporate")
  df <- cer_fetch_auto(url)
  rownames(df) <- NULL
  new_cer_tbl(df,
              source = NGER_LANDING,
              licence = "CC BY 4.0",
              title = paste0("NGER corporate ", year - 1L, "-", year %% 100L))
}

#' NGER electricity sector facility emissions
#'
#' Facility-level Scope 1 emissions and generated electricity for
#' facilities in the electricity generation sector that met the
#' NGER reporting threshold.
#'
#' @param year Integer. Reporting year ending 30 June.
#' @param facility Optional character vector of facility name
#'   substrings (case-insensitive).
#'
#' @return A `cer_tbl` with one row per generator facility.
#'
#' @source Clean Energy Regulator, National Greenhouse and Energy
#'   Register:
#'   <https://cer.gov.au/markets/reports-and-data/nger-reporting-data-and-registers>.
#'   Licensed under CC BY 4.0.
#'
#' @family nger
#' @export
#' @examples
#' \donttest{
#' op <- options(cer.cache_dir = tempdir())
#' try({
#'   e <- cer_nger_electricity(year = 2025, facility = "Bayswater")
#'   head(e)
#' })
#' options(op)
#' }
cer_nger_electricity <- function(year = 2025, facility = NULL) {
  stopifnot(is.numeric(year), length(year) == 1L, year >= 2009L, year <= 2030L)
  url <- nger_urls(year, "electricity")
  df <- cer_fetch_auto(url)
  df <- cer_filter_like(df, "facility_name", facility)
  if (!is.null(facility) && !"facility_name" %in% names(df)) {
    df <- cer_filter_like(df, "facility", facility)
  }
  rownames(df) <- NULL
  new_cer_tbl(df,
              source = NGER_LANDING,
              licence = "CC BY 4.0",
              title = paste0("NGER electricity ", year - 1L, "-", year %% 100L))
}

test_that("cer_safeguard_facilities validates year bounds", {
  expect_error(cer_safeguard_facilities(year = 2010))
  expect_error(cer_safeguard_facilities(year = 2031))
  expect_error(cer_safeguard_facilities(year = "2025"))
})

test_that("nger_urls dispatches on kind arg", {
  expect_error(cer:::nger_urls(2025, "invalid"))
})

test_that("cer_safeguard_facilities live fetch", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("CER_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")

  f <- cer_safeguard_facilities(year = 2025)
  expect_s3_class(f, "cer_tbl")
  expect_gt(nrow(f), 100L)
})

test_that("cer_nger_corporate live fetch", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("CER_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")

  c <- cer_nger_corporate(year = 2025)
  expect_s3_class(c, "cer_tbl")
  expect_gt(nrow(c), 50L)
})

# Citation helper.

#' Cite a cer_tbl or URL in BibTeX, plain-text, or APA form
#'
#' Returns a citation suitable for footnotes, papers, and carbon
#' market analysis reports. Uses the provenance attributes attached
#' to every `cer_tbl`: source URL, licence, retrieval date, title,
#' snapshot pin (from [cer_snapshot()]), and SHA-256 digest.
#'
#' For carbon market research, the BibTeX `note` field includes
#' the snapshot date and first 12 hex characters of the SHA-256,
#' which are the minimum provenance fields needed to defend a
#' published figure against a reviewer.
#'
#' @param x A `cer_tbl` (as returned by any `cer_*` data function)
#'   or a character URL.
#' @param style One of `"text"` (default), `"bibtex"`, or `"apa"`.
#' @param doi Optional DOI (e.g. from [cer_deposit_zenodo()]) to
#'   include in BibTeX output as a `doi` field and APA suffix.
#'
#' @return A character string.
#' @family reproducibility
#' @export
#' @examples
#' x <- data.frame(a = 1)
#' x <- structure(x,
#'   cer_source = "https://cer.gov.au/markets/reports-and-data/accu-project-and-contract-register",
#'   cer_licence = "CC BY 4.0",
#'   cer_retrieved = as.POSIXct("2026-04-24 00:00:00", tz = "UTC"),
#'   cer_title = "ACCU project register",
#'   cer_sha256 = "abc123def456",
#'   cer_snapshot_date = "2026-04-24",
#'   class = c("cer_tbl", "data.frame"))
#' cer_cite(x)
#' cer_cite(x, style = "bibtex")
#' # DOI style: supply any minted DOI (Zenodo, DataCite, etc.).
#' # The placeholder below is illustrative only.
#' cer_cite(x, style = "apa", doi = "10.5281/zenodo.XXXXXXXX")
cer_cite <- function(x, style = c("text", "bibtex", "apa"),
                      doi = NULL) {
  style <- match.arg(style)

  if (is.character(x) && length(x) == 1L) {
    src <- x
    licence <- "CC BY 4.0"
    retrieved <- Sys.time()
    title <- basename(x)
    sha <- NA_character_
    snap <- NA_character_
  } else if (inherits(x, "cer_tbl")) {
    src <- attr(x, "cer_source") %||% ""
    licence <- attr(x, "cer_licence") %||% "CC BY 4.0"
    retrieved <- attr(x, "cer_retrieved") %||% Sys.time()
    title <- attr(x, "cer_title") %||% "Clean Energy Regulator data"
    sha <- attr(x, "cer_sha256") %||% NA_character_
    snap <- attr(x, "cer_snapshot_date") %||% NA_character_
  } else {
    cli::cli_abort(
      "{.arg x} must be a {.cls cer_tbl} or a character URL."
    )
  }

  date_str <- format(as.Date(retrieved), "%Y-%m-%d")
  year <- format(as.Date(retrieved), "%Y")
  sha_str <- if (!is.na(sha) && nzchar(sha)) substr(sha, 1, 12) else ""
  snap_str <- if (!is.na(snap) && nzchar(snap)) snap else ""

  note_parts <- c(
    sprintf("Retrieved %s", date_str),
    sprintf("licensed under %s", licence)
  )
  if (nzchar(snap_str)) note_parts <- c(note_parts,
                                         sprintf("snapshot %s", snap_str))
  if (nzchar(sha_str)) note_parts <- c(note_parts,
                                        sprintf("sha256:%s", sha_str))
  note <- paste(note_parts, collapse = "; ")

  doi_line <- if (!is.null(doi) && nzchar(doi)) {
    sprintf("  doi    = {%s},\n", doi)
  } else {
    ""
  }

  switch(style,
    text = {
      s <- sprintf(
        "Clean Energy Regulator. %s. Retrieved %s from %s. Licensed under %s.",
        title, date_str, src, licence
      )
      if (nzchar(snap_str)) s <- paste0(s, " Snapshot: ", snap_str, ".")
      if (nzchar(sha_str)) s <- paste0(s, " SHA-256: ", sha_str, ".")
      if (!is.null(doi) && nzchar(doi)) s <- paste0(s, " DOI: ", doi, ".")
      s
    },
    apa = {
      s <- sprintf(
        "Clean Energy Regulator. (%s). %s [Data set]. cer.gov.au. %s",
        year, title, src
      )
      if (!is.null(doi) && nzchar(doi)) s <- paste0(s, " https://doi.org/", doi)
      s
    },
    bibtex = sprintf(
      paste0(
        "@misc{cer_%s,\n",
        "  author = {{Clean Energy Regulator}},\n",
        "  title  = {{%s}},\n",
        "  year   = {%s},\n",
        "%s",
        "  note   = {%s},\n",
        "  url    = {%s}\n",
        "}"
      ),
      gsub("[^a-z0-9]+", "_", tolower(title)),
      title, year, doi_line, note, src
    )
  )
}

test_that("cer_cite text includes title and source", {
  x <- structure(data.frame(a = 1),
    cer_source = "https://cer.gov.au/document/x",
    cer_licence = "CC BY 4.0",
    cer_retrieved = as.POSIXct("2026-04-24", tz = "UTC"),
    cer_title = "ACCU project register",
    class = c("cer_tbl", "data.frame")
  )
  s <- cer_cite(x, style = "text")
  expect_match(s, "Clean Energy Regulator")
  expect_match(s, "ACCU project register")
})

test_that("cer_cite bibtex includes SHA and snapshot when set", {
  x <- structure(data.frame(a = 1),
    cer_source = "https://cer.gov.au/document/x",
    cer_licence = "CC BY 4.0",
    cer_retrieved = as.POSIXct("2026-04-24", tz = "UTC"),
    cer_title = "ACCU",
    cer_sha256 = "abcdef123456789012345678",
    cer_snapshot_date = "2026-04-24",
    class = c("cer_tbl", "data.frame")
  )
  b <- cer_cite(x, style = "bibtex")
  expect_match(b, "snapshot 2026-04-24")
  expect_match(b, "sha256:abcdef123456")
})

test_that("cer_cite includes DOI when provided", {
  x <- structure(data.frame(a = 1),
    cer_source = "https://x", cer_title = "t",
    cer_retrieved = Sys.time(),
    class = c("cer_tbl", "data.frame")
  )
  b <- cer_cite(x, style = "bibtex", doi = "10.5281/zenodo.42")
  expect_match(b, "doi    = \\{10\\.5281/zenodo\\.42\\}")
})

# cer_tbl S3 class with provenance header

#' @noRd
new_cer_tbl <- function(df, source = NULL, licence = "CC BY 4.0",
                         retrieved = Sys.time(), title = NULL,
                         sha256 = NA_character_,
                         snapshot_date = cer_snapshot_str()) {
  stopifnot(is.data.frame(df))
  if (is.na(sha256) && !is.null(source) && nzchar(source)) {
    d <- tryCatch(cer_cache_dir(), error = function(e) NULL)
    if (!is.null(d)) {
      ext <- tools::file_ext(source)
      ext <- if (nzchar(ext)) paste0(".", ext) else ""
      f <- file.path(d, paste0(cer_digest_url(source), ext))
      if (file.exists(f)) sha256 <- cer_sha_read(f)
    }
  }
  attr(df, "cer_source") <- source
  attr(df, "cer_licence") <- licence
  attr(df, "cer_retrieved") <- retrieved
  attr(df, "cer_title") <- title
  attr(df, "cer_sha256") <- sha256
  attr(df, "cer_snapshot_date") <- snapshot_date
  class(df) <- c("cer_tbl", class(df))
  df
}

#' Print a cer_tbl
#'
#' Prints a provenance header (title, source URL, licence,
#' retrieval time, dimensions) followed by the data frame.
#'
#' @param x A `cer_tbl` object.
#' @param ... Passed to the next method.
#' @return Invisibly returns `x`.
#' @export
#' @examples
#' x <- data.frame(project_id = "ERF123", accus_issued = 4200)
#' x <- structure(x, cer_title = "Demo", cer_source = "https://cer.gov.au",
#'                cer_licence = "CC BY 4.0", cer_retrieved = Sys.time(),
#'                class = c("cer_tbl", "data.frame"))
#' print(x)
print.cer_tbl <- function(x, ...) {
  title <- attr(x, "cer_title") %||% "Clean Energy Regulator data"
  source <- attr(x, "cer_source") %||% "https://cer.gov.au"
  licence <- attr(x, "cer_licence") %||% "CC BY 4.0"
  retrieved <- attr(x, "cer_retrieved")
  retrieved_str <- if (!is.null(retrieved)) {
    format(retrieved, "%Y-%m-%d %H:%M %Z")
  } else {
    "-"
  }

  sha <- attr(x, "cer_sha256")
  snap <- attr(x, "cer_snapshot_date")
  cat("# cer_tbl: ", title, "\n", sep = "")
  cat("# Source:   ", source, "\n", sep = "")
  cat("# Licence:  ", licence, "\n", sep = "")
  cat("# Retrieved:", retrieved_str, "\n")
  if (!is.null(snap) && !is.na(snap)) {
    cat("# Snapshot: ", snap, "\n", sep = "")
  }
  if (!is.null(sha) && !is.na(sha) && nzchar(sha)) {
    cat("# SHA-256:  ", substr(sha, 1, 16), "...\n", sep = "")
  }
  cat("# Rows: ", formatC(nrow(x), big.mark = ",", format = "d"),
      "  Cols: ", ncol(x), "\n", sep = "")
  cat("\n")

  y <- x
  class(y) <- setdiff(class(y), "cer_tbl")
  print(y, ...)
  invisible(x)
}

test_that("new_cer_tbl attaches provenance attributes", {
  df <- data.frame(a = 1:3, b = letters[1:3])
  x <- cer:::new_cer_tbl(df,
                         source = "https://cer.gov.au",
                         licence = "CC BY 4.0",
                         retrieved = as.POSIXct("2026-04-22 00:00:00", tz = "UTC"),
                         title = "Test")
  expect_s3_class(x, "cer_tbl")
  expect_s3_class(x, "data.frame")
  expect_equal(attr(x, "cer_title"), "Test")
  expect_equal(attr(x, "cer_source"), "https://cer.gov.au")
  expect_equal(attr(x, "cer_licence"), "CC BY 4.0")
})

test_that("new_cer_tbl requires a data frame", {
  expect_error(cer:::new_cer_tbl(list(a = 1)))
})

test_that("print.cer_tbl emits the provenance header", {
  df <- data.frame(a = 1:2)
  x <- cer:::new_cer_tbl(df, title = "Demo",
                         source = "https://example.org",
                         licence = "CC BY 4.0",
                         retrieved = as.POSIXct("2026-01-01 00:00:00", tz = "UTC"))
  out <- capture.output(print(x))
  expect_true(any(grepl("cer_tbl: Demo", out, fixed = TRUE)))
  expect_true(any(grepl("example.org", out, fixed = TRUE)))
  expect_true(any(grepl("CC BY 4.0", out, fixed = TRUE)))
  expect_true(any(grepl("Rows:", out, fixed = TRUE)))
})

test_that("print.cer_tbl returns x invisibly", {
  df <- data.frame(a = 1)
  x <- cer:::new_cer_tbl(df)
  tf <- tempfile()
  sink(tf)
  wv <- withVisible(print(x))
  sink()
  unlink(tf)
  expect_false(wv$visible)
  expect_identical(wv$value, x)
})

# Redirect the cer cache to a per-test-run tempdir so we never
# write into the user's real home filespace. testthat cleans up
# the tempdir automatically when the test session ends; each test
# using its own options() invocation also restores at teardown.
cer_tmp <- tempfile("cer_cache_")
dir.create(cer_tmp, recursive = TRUE, showWarnings = FALSE)
options(cer.cache_dir = cer_tmp)

# Safeguard Mechanism reform (1 July 2023) handling.
#
# The reform changed the baseline regime fundamentally:
# - Pre-reform (2016-17 to 2022-23): production-adjusted,
#   facility-specific baselines; no SMCs.
# - Post-reform (2023-24 onward): industry-average default baselines
#   declining 4.9% per annum (nominal) to 2030; SMC (Safeguard
#   Mechanism Credit) instrument introduced.
#
# Time-series analysis across the reform without flagging the
# regime break is a common analytical error.

#' Safeguard Mechanism reform-era constants
#'
#' @noRd
SAFEGUARD_REFORM_DATE <- as.Date("2023-07-01")

#' @noRd
SAFEGUARD_ANNUAL_DECLINE_RATE <- 0.049

#' Classify a Safeguard record by regime (pre- or post-reform)
#'
#' @param financial_year Character or integer year; accepts
#'   `"2022-23"`, `2022`, etc.
#' @return `"pre_reform"` (FY 2022-23 or earlier) or
#'   `"post_reform"` (FY 2023-24 or later).
#' @noRd
cer_safeguard_regime <- function(financial_year) {
  # Accept integers, character "YYYY-YY", or "YYYY/YY" etc.
  start_year <- if (is.numeric(financial_year)) {
    as.integer(financial_year)
  } else {
    as.integer(sub("[-/].*", "", as.character(financial_year)))
  }
  ifelse(start_year < 2023, "pre_reform", "post_reform")
}

#' Compute a declining Safeguard baseline trajectory
#'
#' Implements the 4.9% per-annum nominal decline in industry-average
#' default baselines introduced by the 2023 reform. The formula is:
#'
#' \deqn{B_t = B_{2023\text{-}24} \times (1 - 0.049)^{t - 2023}}
#'
#' where `t` is the starting calendar year of the financial year.
#' Default values are drawn from
#' `inst/extdata/safeguard_industry_baselines.csv`.
#'
#' @param industry Character scalar matching an `industry` value
#'   from [cer_safeguard_industry_baselines()].
#' @param from_year Start year (default 2023 for FY 2023-24).
#' @param to_year End year (default 2029 for FY 2029-30).
#'
#' @return A data frame with `year`, `financial_year`, `baseline`,
#'   and `decline_factor` columns.
#'
#' @references
#' Commonwealth of Australia (2023). \emph{Safeguard Mechanism
#'   (Crediting) Amendment Act 2023}. Introduces 4.9% p.a. nominal
#'   baseline decline to 2030.
#'
#' Clean Energy Regulator (annual). \emph{Safeguard Mechanism:
#'   declining baselines}. Methodology notes accompanying annual
#'   baseline determinations.
#'
#' @family safeguard
#' @export
#' @examples
#' cer_safeguard_baseline_trajectory("Aluminium smelting")
#' cer_safeguard_baseline_trajectory("Cement clinker",
#'                                    from_year = 2023, to_year = 2030)
cer_safeguard_baseline_trajectory <- function(industry,
                                                from_year = 2023,
                                                to_year = 2030) {
  stopifnot(is.character(industry), length(industry) == 1L)
  b <- cer_safeguard_industry_baselines()
  hit <- b[tolower(b$industry) == tolower(industry), ]
  if (nrow(hit) == 0L) {
    cli::cli_abort(c(
      "Industry {.val {industry}} not found in bundled baseline table.",
      "i" = "Run {.code cer_safeguard_industry_baselines()} to see available industries."
    ))
  }
  b_base <- hit$baseline_2023_24_t_co2e_per_unit[1L]
  years <- seq.int(from_year, to_year)
  decline <- (1 - SAFEGUARD_ANNUAL_DECLINE_RATE) ^ (years - 2023L)
  out <- data.frame(
    year           = years,
    financial_year = sprintf("%d-%02d", years, (years + 1L) %% 100L),
    baseline       = b_base * decline,
    decline_factor = decline,
    stringsAsFactors = FALSE
  )
  out
}

#' Bundled Safeguard industry-average default baselines
#'
#' @return A `cer_tbl` of industry default baselines.
#' @family safeguard
#' @export
#' @examples
#' cer_safeguard_industry_baselines()
cer_safeguard_industry_baselines <- function() {
  path <- system.file("extdata", "safeguard_industry_baselines.csv",
                       package = "cer")
  df <- utils::read.csv(path, stringsAsFactors = FALSE,
                         na.strings = c("", "NA"), check.names = FALSE)
  new_cer_tbl(df,
              source = "https://cer.gov.au/schemes/safeguard-mechanism",
              licence = "CC BY 4.0",
              title = "Safeguard industry-average default baselines")
}

#' Trade-exposed baseline-adjusted (TEBA) facility list
#'
#' Returns the bundled list of facilities declared as TEBA under
#' the 2023 reform. TEBA facilities are eligible for softer
#' baseline treatment reflecting international competitiveness.
#'
#' @return A `cer_tbl` of TEBA facilities.
#'
#' @references
#' Commonwealth of Australia (2023). \emph{National Greenhouse
#'   and Energy Reporting (Safeguard Mechanism) Rule 2015} as
#'   amended by the Safeguard Mechanism (Crediting) Amendment
#'   Rule 2023. TEBA definition and declaration process.
#'
#' @family safeguard
#' @export
#' @examples
#' cer_safeguard_teba_facilities()
cer_safeguard_teba_facilities <- function() {
  path <- system.file("extdata", "safeguard_teba_facilities.csv",
                       package = "cer")
  df <- utils::read.csv(path, stringsAsFactors = FALSE,
                         na.strings = c("", "NA"), check.names = FALSE)
  new_cer_tbl(df,
              source = "https://cer.gov.au/markets/reports-and-data/safeguard-data/trade-exposed-baseline-adjusted-facility-data",
              licence = "CC BY 4.0",
              title = "Safeguard TEBA facilities")
}

test_that("cer_safeguard_regime classifies correctly", {
  cer_ns <- asNamespace("cer")
  expect_equal(cer_ns$cer_safeguard_regime(2022L), "pre_reform")
  expect_equal(cer_ns$cer_safeguard_regime(2023L), "post_reform")
  expect_equal(cer_ns$cer_safeguard_regime("2022-23"), "pre_reform")
  expect_equal(cer_ns$cer_safeguard_regime("2023-24"), "post_reform")
})

# Level 2: base year baseline = input baseline.
test_that("baseline at 2023 equals bundled value (identity)", {
  traj <- cer_safeguard_baseline_trajectory("Aluminium smelting",
                                             from_year = 2023,
                                             to_year = 2023)
  expect_equal(traj$decline_factor, 1, tolerance = 1e-12)
  expect_equal(traj$baseline, 1.650, tolerance = 1e-6)
})

# Level 2: 2029-30 baseline = 2023-24 * (1-0.049)^6.
test_that("2029 baseline matches 4.9% p.a. declining formula", {
  traj <- cer_safeguard_baseline_trajectory("Aluminium smelting")
  b_2029 <- traj$baseline[traj$year == 2029]
  expected <- 1.650 * (1 - 0.049)^6
  expect_equal(b_2029, expected, tolerance = 1e-6)
})

# Level 3: monotonic decline.
test_that("baseline is monotone-decreasing across years", {
  traj <- cer_safeguard_baseline_trajectory("Cement clinker")
  expect_true(all(diff(traj$baseline) < 0))
})

test_that("cer_safeguard_industry_baselines returns bundled csv", {
  b <- cer_safeguard_industry_baselines()
  expect_s3_class(b, "data.frame")
  expect_gt(nrow(b), 15L)
})

test_that("cer_safeguard_teba_facilities returns bundled csv", {
  t <- cer_safeguard_teba_facilities()
  expect_s3_class(t, "data.frame")
  expect_true("facility_name" %in% names(t))
})

test_that("baseline trajectory errors on unknown industry", {
  expect_error(
    cer_safeguard_baseline_trajectory("not an industry"),
    "not found"
  )
})

# Session-level manifest: every fetch appends a row recording URL,
# SHA-256, timestamp, snapshot pin, and dataset title. Export for
# paper appendix or Zenodo deposit.

#' @noRd
.cer_manifest_append <- function(url, file = NA_character_,
                                  title = NA_character_,
                                  licence = NA_character_) {
  sha <- if (!is.na(file) && file.exists(file)) cer_sha256(file) else NA_character_
  size <- if (!is.na(file) && file.exists(file)) {
    as.numeric(file.info(file)$size)
  } else {
    NA_real_
  }
  row <- data.frame(
    url            = url,
    title          = title,
    licence        = licence,
    sha256         = sha,
    size_bytes     = size,
    retrieved      = format(Sys.time(), "%Y-%m-%dT%H:%M:%S%z"),
    snapshot_date  = cer_snapshot_str(),
    r_version      = as.character(getRversion()),
    cer_version    = as.character(utils::packageVersion("cer")),
    stringsAsFactors = FALSE
  )
  key <- url
  existing <- .cer_env$manifest %||% list()
  existing[[key]] <- row
  .cer_env$manifest <- existing
  invisible(row)
}

#' Return the session manifest of fetched CER datasets
#'
#' Every call to a data function appends one row to the session
#' manifest, recording URL, dataset title, SHA-256 of the cached
#' file, size, retrieval timestamp, and the snapshot pin set via
#' [cer_snapshot()]. Duplicate URLs within a session are
#' deduplicated (last fetch wins).
#'
#' For carbon market research this is the minimum artefact needed
#' for reproducibility: ACCU project registers and Safeguard
#' releases change retroactively, so a citation that points to a
#' URL alone is not enough. The manifest plus SHA-256 fixes the
#' exact bytes analysed.
#'
#' @param format One of `"df"` (default), `"yaml"`, or `"json"`.
#' @return A data frame, YAML string, or JSON string.
#' @family reproducibility
#' @export
#' @examples
#' \donttest{
#' op <- options(cer.cache_dir = tempdir())
#' cer_manifest_clear()
#' cer_snapshot("2026-04-24")
#' cer_manifest()
#' options(op)
#' }
cer_manifest <- function(format = c("df", "yaml", "json")) {
  format <- match.arg(format)
  rows <- .cer_env$manifest %||% list()
  if (length(rows) == 0L) {
    df <- data.frame(
      url = character(0), title = character(0),
      licence = character(0), sha256 = character(0),
      size_bytes = numeric(0), retrieved = character(0),
      snapshot_date = character(0), r_version = character(0),
      cer_version = character(0),
      stringsAsFactors = FALSE
    )
  } else {
    df <- do.call(rbind, rows)
    rownames(df) <- NULL
  }
  if (format == "df") return(df)
  if (format == "json") {
    if (!requireNamespace("jsonlite", quietly = TRUE)) {
      cli::cli_abort("JSON output requires the {.pkg jsonlite} package.")
    }
    return(jsonlite::toJSON(df, pretty = TRUE, auto_unbox = TRUE,
                            na = "null"))
  }
  cer_to_yaml(df)
}

#' Clear the session manifest
#'
#' @return Invisibly `NULL`.
#' @family reproducibility
#' @export
#' @examples
#' cer_manifest_clear()
cer_manifest_clear <- function() {
  .cer_env$manifest <- list()
  invisible(NULL)
}

#' Write the session manifest to a file
#'
#' @param path Output file path. Extension determines format when
#'   `format = "auto"`.
#' @param format One of `"auto"`, `"csv"`, `"yaml"`, `"json"`.
#' @return Invisibly, the absolute path written.
#' @family reproducibility
#' @export
#' @examples
#' \donttest{
#' p <- tempfile(fileext = ".csv")
#' cer_manifest_clear()
#' cer_manifest_write(p)
#' }
cer_manifest_write <- function(path,
                                format = c("auto", "csv", "yaml", "json")) {
  format <- match.arg(format)
  if (format == "auto") {
    ext <- tolower(tools::file_ext(path))
    format <- switch(ext,
      csv = "csv",
      yaml = "yaml", yml = "yaml",
      json = "json",
      "csv"
    )
  }
  if (format == "csv") {
    utils::write.csv(cer_manifest("df"), path, row.names = FALSE)
  } else if (format == "yaml") {
    writeLines(cer_manifest("yaml"), path)
  } else {
    writeLines(cer_manifest("json"), path)
  }
  invisible(normalizePath(path, mustWork = FALSE))
}

#' @noRd
cer_to_yaml <- function(df) {
  if (nrow(df) == 0L) return("# empty manifest\n")
  entries <- vapply(seq_len(nrow(df)), function(i) {
    lines <- vapply(names(df), function(k) {
      v <- df[i, k, drop = TRUE]
      v <- if (is.na(v)) "null" else {
        s <- as.character(v)
        if (grepl("[:#\\-\\[\\]{}!&*]|^ | $", s)) {
          paste0("\"", gsub("\"", "\\\"", s, fixed = TRUE), "\"")
        } else {
          s
        }
      }
      sprintf("  %s: %s", k, v)
    }, character(1))
    paste0("- ", sub("^  ", "", lines[1]), "\n",
           paste(lines[-1], collapse = "\n"))
  }, character(1))
  paste(entries, collapse = "\n")
}

test_that("cer_manifest_clear empties the manifest", {
  cer_manifest_clear()
  df <- cer_manifest("df")
  expect_s3_class(df, "data.frame")
  expect_equal(nrow(df), 0L)
})

test_that("cer_manifest df has expected columns", {
  cer_manifest_clear()
  df <- cer_manifest("df")
  expect_true(all(c("url", "title", "licence", "sha256", "size_bytes",
                    "retrieved", "snapshot_date", "r_version",
                    "cer_version") %in% names(df)))
})

test_that("cer_manifest_write writes CSV", {
  cer_manifest_clear()
  p <- tempfile(fileext = ".csv")
  cer_manifest_write(p)
  expect_true(file.exists(p))
  df <- utils::read.csv(p)
  expect_s3_class(df, "data.frame")
})

test_that("manifest records a fetch via internal helper", {
  cer_manifest_clear()
  f <- tempfile()
  writeLines("test", f)
  cer_ns <- asNamespace("cer")
  cer_ns$.cer_manifest_append(
    url = "https://cer.gov.au/document/x",
    file = f,
    title = "test dataset"
  )
  df <- cer_manifest("df")
  expect_equal(nrow(df), 1L)
  expect_equal(df$title, "test dataset")
  expect_true(nzchar(df$sha256))
})

# International comparator and carbondata interop.

#' Remap an ACCU project data frame to the carbondata voluntary-
#' project schema
#'
#' The `carbondata` R package exposes Verra, Gold Standard, ACR,
#' and CAR voluntary projects under a unified schema. This function
#' renames ACCU register columns to match, so an analyst can
#' compare ACCU issuance patterns against international voluntary
#' markets on a consistent basis.
#'
#' Schema mapping is lossy: ACCU-specific fields (Chubb flag,
#' permanence period, crediting-period dates) pass through with
#' original names.
#'
#' @param x A `cer_tbl` from [cer_accu_projects()].
#'
#' @return A data frame with carbondata-style columns:
#'   `project_id`, `project_name`, `methodology`, `project_type`,
#'   `country`, `registry`, `issued_credits_to_date`,
#'   `retired_credits_to_date`.
#'
#' @family interop
#' @export
#' @examples
#' \donttest{
#' op <- options(cer.cache_dir = tempdir())
#' try({
#'   ccy <- cer_accu_projects()
#'   carb <- cer_to_carbondata(ccy)
#'   head(carb)
#' })
#' options(op)
#' }
cer_to_carbondata <- function(x) {
  df <- as.data.frame(x, stringsAsFactors = FALSE)

  rename_first <- function(df, candidates, target) {
    hit <- intersect(candidates, names(df))
    if (length(hit) > 0L) names(df)[names(df) == hit[1L]] <- target
    df
  }

  df <- rename_first(df, c("project_id", "era_project_id", "erf_project_id"),
                     "project_id")
  df <- rename_first(df, c("project_name", "name"), "project_name")
  df <- rename_first(df, c("method"), "methodology")
  df <- rename_first(df, c("sector", "method_sector"), "project_type")
  df <- rename_first(df, c("accus_issued", "issued"),
                     "issued_credits_to_date")
  df <- rename_first(df, c("accus_surrendered", "accus_relinquished",
                           "relinquished"),
                     "retired_credits_to_date")

  df$country <- "Australia"
  df$registry <- "Clean Energy Regulator (ACCU)"
  df
}

#' Bundled international carbon market comparator
#'
#' Returns headline price/volume figures for comparison between
#' ACCU (Australia) and major international markets: EU ETS, New
#' Zealand ETS, California-Quebec (WCI), Verra VCUs, and Gold
#' Standard. Values are indicative snapshots. For live data use
#' the companion `carbondata` package.
#'
#' @return A `cer_tbl` of indicative comparator data.
#'
#' @references
#' International Carbon Action Partnership (annual). \emph{Emissions
#'   Trading Worldwide: Status Report}. \url{https://icapcarbonaction.com/en}
#'
#' World Bank (annual). \emph{State and Trends of Carbon Pricing}.
#'
#' @family interop
#' @export
#' @examples
#' cer_international_comparator()
cer_international_comparator <- function() {
  df <- data.frame(
    market = c("ACCU", "EU_ETS", "NZ_ETS", "WCI",
               "Verra_VCU", "Gold_Standard_VER"),
    jurisdiction = c("Australia", "European Union", "New Zealand",
                      "California+Quebec", "International",
                      "International"),
    unit_name = c("ACCU", "EUA", "NZU", "CCA",
                   "VCU", "VER"),
    instrument_type = c("Offset (government-purchased or voluntary)",
                         "Compliance allowance",
                         "Compliance unit (hybrid cap and offset)",
                         "Compliance allowance",
                         "Voluntary offset",
                         "Voluntary offset"),
    indicative_price_2024_usd = c(22, 75, 35, 40, 5, 10),
    reference = c("QCMR Q4 2024",
                   "EEX secondary market",
                   "NZEA auction clearing price",
                   "WCI Joint Auction",
                   "Berkeley VROD",
                   "Gold Standard registry"),
    stringsAsFactors = FALSE
  )
  new_cer_tbl(df,
              source = "https://icapcarbonaction.com/en",
              licence = "Mixed (see reference column)",
              title = "International carbon market comparator (indicative)")
}

test_that("cer_to_carbondata renames known columns", {
  fake <- data.frame(
    project_id = "ERF101",
    project_name = "Test project",
    method = "HIR",
    accus_issued = 10000,
    stringsAsFactors = FALSE
  )
  out <- cer_to_carbondata(fake)
  expect_true("methodology" %in% names(out))
  expect_true("issued_credits_to_date" %in% names(out))
  expect_equal(out$country, "Australia")
  expect_equal(out$registry, "Clean Energy Regulator (ACCU)")
})

test_that("cer_international_comparator returns ACCU row", {
  comp <- cer_international_comparator()
  expect_s3_class(comp, "data.frame")
  expect_true("ACCU" %in% comp$market)
  expect_true("EU_ETS" %in% comp$market)
})

# LRET + SRES

#' @noRd
LRET_POWER_STATIONS_CSV <- "https://cer.gov.au/document/historical-accredited-power-stations-and-projects-0"

#' @noRd
LRET_LANDING <- "https://cer.gov.au/markets/reports-and-data/large-scale-renewable-energy-data/historical-large-scale-renewable-energy-supply-data"

#' @noRd
SRES_INSTALL_XLSX <- "https://cer.gov.au/document/sres-postcode-data-installations-2011-to-present-and-totals"

#' @noRd
SRES_CAPACITY_XLSX <- "https://cer.gov.au/document/sres-postcode-data-capacity-2011-to-present-and-totals"

#' @noRd
SRES_LANDING <- "https://cer.gov.au/markets/reports-and-data/small-scale-installation-postcode-data"

#' LRET accredited power stations
#'
#' Power stations accredited under the Large-scale Renewable
#' Energy Target ('LRET') and eligible to create Large-scale
#' Generation Certificates ('LGCs').
#'
#' @param technology Optional character vector of technology
#'   substrings (e.g. `"solar"`, `"wind"`, `"hydro"`, `"bioenergy"`).
#' @param state Optional character vector of Australian state codes
#'   (e.g. `c("VIC", "SA")`).
#'
#' @return A `cer_tbl` with one row per accredited power station,
#'   including accreditation date, technology, capacity (MW), and
#'   state.
#'
#' @source Clean Energy Regulator, Historical large-scale renewable
#'   energy supply data:
#'   <https://cer.gov.au/markets/reports-and-data/large-scale-renewable-energy-data/historical-large-scale-renewable-energy-supply-data>.
#'   Licensed under CC BY 4.0.
#'
#' @references
#' Commonwealth of Australia. \emph{Renewable Energy (Electricity)
#'   Act 2000}. Establishes the Renewable Energy Target (RET), the
#'   LGC (Large-scale Generation Certificate) and the shortfall
#'   charge mechanism.
#'
#' Clean Energy Regulator (annual). \emph{LRET: Accreditation and
#'   eligibility guidelines for power stations}.
#'
#' Australian Energy Market Commission (2023). \emph{Integrated
#'   System Plan 2024} reference on renewable capacity expansion.
#'
#' @family lret
#' @export
#' @examples
#' \donttest{
#' op <- options(cer.cache_dir = tempdir())
#' s <- cer_lgc_power_stations(technology = "solar", state = "VIC")
#' head(s)
#' options(op)
#' }
cer_lgc_power_stations <- function(technology = NULL, state = NULL) {
  df <- cer_fetch_csv(LRET_POWER_STATIONS_CSV)
  df <- cer_filter_like(df, "fuel_source", technology)
  if (is.null(technology) == FALSE && !"fuel_source" %in% names(df)) {
    df <- cer_filter_like(df, "technology", technology)
  }
  df <- cer_filter_in(df, "state", state)
  rownames(df) <- NULL
  new_cer_tbl(df,
              source = LRET_LANDING,
              licence = "CC BY 4.0",
              title = "LRET accredited power stations")
}

#' SRES small-scale installations by postcode
#'
#' Monthly Small-scale Renewable Energy Scheme ('SRES') installation
#' counts or capacity by postcode. Covers rooftop solar PV, solar
#' water heaters, air-source heat pumps, and small batteries.
#'
#' @param technology Character. One of `"solar_pv"` (default), `"swh"`
#'   (solar water heater), `"heat_pump"`, or `"battery"`.
#' @param postcode Optional character vector of 4-digit Australian
#'   postcodes to filter on.
#' @param measure Character. `"installations"` (default, counts) or
#'   `"capacity"` (total kW installed).
#'
#' @return A `cer_tbl`. For `measure = "installations"` columns
#'   include postcode and per-month install counts. For
#'   `measure = "capacity"`, per-month kW capacity.
#'
#' @source Clean Energy Regulator, Small-scale installation
#'   postcode data:
#'   <https://cer.gov.au/markets/reports-and-data/small-scale-installation-postcode-data>.
#'   Licensed under CC BY 4.0.
#'
#' @references
#' Commonwealth of Australia. \emph{Renewable Energy (Electricity)
#'   Act 2000}, Small-scale Renewable Energy Scheme provisions.
#'
#' Australian Energy Market Operator (2024). \emph{Distributed PV
#'   Forecasting Methodology}. AEMO uses SRES installation data
#'   as input to ISP and ESOO forecasts.
#'
#' Best, R., Burke, P.J. and Nishitateno, S. (2019).
#'   "Understanding the determinants of rooftop solar
#'   installation: evidence from household surveys in Australia."
#'   \emph{Australian Journal of Agricultural and Resource
#'   Economics}, 63(4), 922-939.
#'   \doi{10.1111/1467-8489.12319}
#'
#' @family sres
#' @export
#' @examples
#' \donttest{
#' op <- options(cer.cache_dir = tempdir())
#' inst <- cer_sres_installations(postcode = c("2000", "3000"))
#' head(inst)
#' options(op)
#' }
cer_sres_installations <- function(technology = c("solar_pv", "swh", "heat_pump", "battery"),
                                    postcode = NULL,
                                    measure = c("installations", "capacity")) {
  technology <- match.arg(technology)
  measure <- match.arg(measure)

  url <- if (measure == "installations") SRES_INSTALL_XLSX else SRES_CAPACITY_XLSX
  df <- cer_fetch_xlsx(url, sheet = 1, skip = 0)

  tech_col <- intersect(c("sgu_type", "technology", "type"), names(df))[1]
  if (!is.na(tech_col)) {
    label <- switch(technology,
      solar_pv = "solar",
      swh = "water",
      heat_pump = "heat pump",
      battery = "battery"
    )
    df <- cer_filter_like(df, tech_col, label)
  }

  if (!is.null(postcode)) {
    pc_col <- intersect(c("postcode", "post_code"), names(df))[1]
    if (!is.na(pc_col)) {
      df <- df[as.character(df[[pc_col]]) %in% as.character(postcode), , drop = FALSE]
    }
  }
  rownames(df) <- NULL

  new_cer_tbl(df,
              source = SRES_LANDING,
              licence = "CC BY 4.0",
              title = paste0("SRES ", technology, " ", measure))
}

test_that("cer_sres_installations validates technology arg", {
  expect_error(cer_sres_installations(technology = "nuclear"))
})

test_that("cer_sres_installations validates measure arg", {
  expect_error(cer_sres_installations(measure = "kwh"))
})

test_that("cer_lgc_power_stations live fetch", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("CER_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")

  s <- cer_lgc_power_stations()
  expect_s3_class(s, "cer_tbl")
  expect_gt(nrow(s), 50L)
})

test_that("cer_sres_installations live fetch", {
  skip_on_cran()
  skip_if_offline()
  skip_if(Sys.getenv("CER_LIVE_TESTS") != "true",
          "Live network tests disabled by default.")

  inst <- cer_sres_installations(technology = "solar_pv", postcode = "2000")
  expect_s3_class(inst, "cer_tbl")
})

---
title: "RET compliance primer: LGCs, STCs, and the shortfall charge"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{RET compliance primer: LGCs, STCs, and the shortfall charge}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", eval = FALSE)
```

The Renewable Energy Target (RET) is Australia's two-scheme
renewable certificate framework:

- **LRET** (Large-scale): accredited power stations create LGCs
  (Large-scale Generation Certificates). 1 LGC = 1 MWh.
- **SRES** (Small-scale): small-scale systems (rooftop solar,
  solar water heaters, heat pumps) create STCs (Small-scale
  Technology Certificates). STC multipliers vary by system type
  and zone.

Liable entities (mostly electricity retailers) must surrender
LGCs to meet the annual LRET target and STCs to meet the annual
small-scale percentage. The **shortfall charge** caps the price:
AUD 65 per MWh (pre-tax) for LGCs; AUD 40 per STC.

## LRET: accredited power stations

```{r}
library(cer)
cer_snapshot("2026-04-24")

stations <- cer_lgc_power_stations(technology = "solar")
head(stations[, c("accredited_power_station", "state",
                   "capacity_mw", "accreditation_date")])
```

## SRES: postcode installations

```{r}
sres <- cer_sres_installations(measure = "installations")
head(sres[, 1:8])
```

## Quick compliance check

A retailer with 10 TWh (= 10,000,000 MWh) of liable acquisitions
and an LRET target of 18 per cent must surrender 1,800,000 LGCs.
If they surrender 1,700,000 and no grandfathered arrangements
apply, the shortfall is 100,000 LGCs, and the shortfall charge is
AUD 6.5m (pre-tax).

```{r}
liable_mwh <- 10e6
target_pct <- 0.18
lgcs_required <- liable_mwh * target_pct
lgcs_surrendered <- 1700000

shortfall_lgcs <- max(0, lgcs_required - lgcs_surrendered)
shortfall_aud <- shortfall_lgcs * 65   # pre-tax shortfall charge
shortfall_aud
```

## Reconciling against QCMR headline

```{r}
# Cumulative LGC issuances
cer_reconcile(
  value    = sum(stations$capacity_mw, na.rm = TRUE) * 1e3,  # rough
  quarter  = "2024-Q4",
  measure  = "lgc_cumulative_issuances_millions"
)
```

The rough check using capacity is a weak proxy; real reconciliation
would use the published LGC issuance register. This vignette is
illustrative of the reconciliation pattern.

---
title: "Offsets integrity primer: reading ACCUs after the Chubb Review"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Offsets integrity primer: reading ACCUs after the Chubb Review}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", eval = FALSE)
```

Australian Carbon Credit Units (ACCUs) are not fungible after
Chubb (2022). Different methods have different integrity profiles;
some methods are under active review by the Independent Integrity
Assurance Committee; some are suspended. A headline figure like
"185 million ACCUs issued to date" is only meaningful once
decomposed by integrity tier.

This vignette walks through the `cer` integrity layer.

## What changed after Chubb (2022)

The Independent Review of Australian Carbon Credit Units (Chubb,
December 2022) identified five ACCU methods for specific attention:

- **Human-Induced Regeneration (HIR).** Recommended replacement by
  a new Integrated Farm and Land Management (IFLM) method.
- **Avoided Deforestation.** Closed to new project registrations.
- **Landfill Gas.** Tightened baseline-crediting expectations.
- **Soil Carbon.** Strengthened Measurement, Reporting, and
  Verification (MRV) requirements.
- **Plantings (2014 method).** Superseded by the 2024 FullCAM
  method.

Independent academic critique (Macintosh et al. 2022, ANU) was
sharper on HIR specifically. The live debate is continuing.

## Query the integrity scorecard

```{r}
library(cer)
cer_snapshot("2026-04-24")

all_methods <- cer_method_integrity()
all_methods[, c("method_short", "status", "integrity_tier", "chubb_affected")]
```

## Filter to contested methods

```{r}
contested <- cer_method_integrity(tier = "contested")
contested[, c("method_short", "method_full_name", "status")]
```

## Aggregating ACCUs by integrity tier

```{r}
projects <- cer_accu_projects(status = "all")

by_tier <- cer_accu_aggregate(projects,
                               value_col = "accus_issued",
                               warn_contested_pct = 0.05)
by_tier
```

The function emits a warning when contested-integrity methods
account for more than 5 per cent of the total : a signal that a
single headline figure is not appropriate.

## Warnings when querying a suspended method

```{r}
# Will emit a post-Chubb integrity warning
ad <- cer_accu_projects(method = "avoided deforestation",
                         status = "all")
```

## Linking to the authoritative determination

```{r}
cer_method_determination("HIR")
cer_legislative_instrument("F2019C00664")
```

## Rule of thumb

If your analysis depends on an aggregate ACCU figure, report three
numbers: total, high-integrity share, and contested share. A
reviewer will ask for this breakdown; better to have it in the
table than in a footnote.

---
title: "Methodology: hand-computed worked examples"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Methodology: hand-computed worked examples}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(cer)
```

Every computation in `cer` is verifiable with pen and paper. This
vignette walks through the three formula functions.

## 1. Safeguard declining baseline

### Formula

\[
B_t \;=\; B_{2023\text{-}24} \times (1 - 0.049)^{t - 2023}
\]

Source: Safeguard Mechanism (Crediting) Amendment Act 2023.

### Hand calculation

Aluminium smelting 2023-24 default baseline = 1.650 t CO2e per
tonne aluminium. For FY 2029-30 (t = 2029):

\[
B_{2029\text{-}30} = 1.650 \times (1 - 0.049)^6 = 1.650 \times 0.738 = 1.218
\]

### Package output

```{r}
traj <- cer_safeguard_baseline_trajectory("Aluminium smelting",
                                           from_year = 2023,
                                           to_year = 2029)
traj
```

Row for 2029: baseline ~= 1.218. Matches.

### Sanity check at base year

```{r}
traj[traj$year == 2023, ]
```

Decline factor = 1.000; baseline = 1.650. Identity holds.

## 2. QCMR reconciliation

### Formula

\[
\pi \;=\; \frac{V - R}{R}
\]

where V is the live-fetched aggregate and R the QCMR reference.

### Hand calculation

ACCU cumulative issuances at end of 2024-Q4 per QCMR = 185,000,000.
Suppose a user's fetch shows 184,200,000 (0.43 per cent low).

\[
\pi = \frac{184{,}200{,}000 - 185{,}000{,}000}{185{,}000{,}000}
     = \frac{-800{,}000}{185{,}000{,}000} = -0.00432 = -0.432\%
\]

### Package output

```{r}
cer_reconcile(value   = 184200000,
              quarter = "2024-Q4",
              measure = "accu_cumulative_issuances")
```

At -0.43 per cent the reconciliation falls within the default
2 per cent warning threshold; no warning fires.

## 3. SHA-256 integrity

### Specification

SHA-256 as defined in NIST FIPS 180-4. Empty-string digest:

```
e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855
```

### Verification

```{r}
f <- tempfile()
file.create(f)
cer_sha256(f)
```

Matches the NIST reference.

## Summary

| Function | Formula | Verified |
|---|---|---|
| `cer_safeguard_baseline_trajectory` | B_t = B_0 * (1 - 0.049)^n | Yes |
| `cer_reconcile` | (V - R) / R | Yes |
| `cer_sha256` | NIST FIPS 180-4 | Yes |

Any disagreement between a hand calculation and the package output
is a bug; please file an issue.

---
title: "Safeguard Mechanism reform: pre- vs post-1 July 2023"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Safeguard Mechanism reform: pre- vs post-1 July 2023}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", eval = FALSE)
```

On 1 July 2023 the Safeguard Mechanism shifted from
production-adjusted facility-specific baselines to industry-average
default baselines declining 4.9 per cent per annum (nominal) to
2030. The *Safeguard Mechanism (Crediting) Amendment Act 2023*
also introduced Safeguard Mechanism Credits (SMCs), a new
crediting instrument. Time-series analysis across the reform
without flagging the regime break is a common analytical error.

## The regime flag

```{r}
library(cer)
cer_snapshot("2026-04-24")

fac_22_23 <- cer_safeguard_facilities(year = 2023)
fac_23_24 <- cer_safeguard_facilities(year = 2024)

unique(fac_22_23$regime)
unique(fac_23_24$regime)
```

The `regime` column is `"pre_reform"` for FY ending in 2023 or
earlier; `"post_reform"` from FY 2023-24 onward.

## Declining baseline trajectory

```{r}
traj <- cer_safeguard_baseline_trajectory("Aluminium smelting",
                                            from_year = 2023,
                                            to_year = 2030)
traj
```

In 2029-30, the aluminium smelting default baseline is ~27 per
cent below its 2023-24 value (`(1 - 0.049)^6 ~= 0.738`). This is
the key number for emissions-intensity reform analysis.

## TEBA facilities

```{r}
teba <- cer_safeguard_teba_facilities()
head(teba)

# Safeguard data merged with TEBA declaration status
sum(fac_23_24$teba_declared, na.rm = TRUE)
```

Trade-Exposed Baseline-Adjusted (TEBA) facilities are eligible for
softer baseline treatment where international competition would
otherwise lead to emissions-leakage.

## Analytical pitfall: summing across regimes

```{r}
# DO NOT do this without a regime-specific annotation:
all_years <- rbind(fac_22_23, fac_23_24)
# If you sum covered_emissions across all_years you are combining
# two different accounting regimes. Split by regime first:
aggregate(covered_emissions ~ regime, data = all_years, FUN = sum,
          na.rm = TRUE)
```

## SMCs post-reform

SMCs did not exist pre-reform. In post-reform data, the SMC columns
count issuances and surrenders for facilities operating below
their baseline.

```{r}
# Post-reform SMC issuances (approximation; column names may drift)
sum(fac_23_24[, grep("smc", names(fac_23_24))], na.rm = TRUE)
```

## Reconciling against the QCMR

```{r}
total_23_24 <- sum(fac_23_24$covered_emissions, na.rm = TRUE)
cer_reconcile(
  value   = total_23_24 / 1e6,  # convert t to Mt
  quarter = "2023-24",
  measure = "safeguard_covered_emissions_mt"
)
```

If the reconciliation gap exceeds 2 per cent, a CER restatement
has likely occurred between your snapshot date and the bundled
QCMR reference.

---
title: "NGER scope reconciliation: Scope 1 vs Scope 2 vs Climate Active"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{NGER scope reconciliation: Scope 1 vs Scope 2 vs Climate Active}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>", eval = FALSE)
```

Scope confusion is the single most common error in Australian
emissions reporting. NGER publishes:

- **Scope 1** (direct emissions) at the corporate level.
- **Scope 2** (indirect from purchased energy) at the corporate
  level.
- **Scope 1 at facility level** for the electricity generation
  sector only.

NGER does not cover Scope 3. Climate Active (Australia's voluntary
carbon-neutral certification) requires Scope 3 separately, with
location-based and market-based Scope 2 distinction.

## Explicit scope selection

```{r}
library(cer)
cer_snapshot("2026-04-24")

corp <- cer_nger_corporate(year = 2025)

scope1 <- cer_nger_scope(corp, scope = "1")
head(scope1[, c("reporting_corporation", "emissions_t_co2e", "scope")])

scope12 <- cer_nger_scope(corp, scope = "1_plus_2")
sum(scope12$emissions_t_co2e, na.rm = TRUE)
```

If the scope column is missing (NGER renames columns year to year),
the error message lists available columns so you can pass
`col_pattern` explicitly.

## Climate Active translator

```{r}
ca <- cer_nger_climate_active(corp)
ca_cols <- grep("operational_scope", names(ca), value = TRUE)
head(ca[, c("reporting_corporation", ca_cols)])
```

The renamed columns match a typical Climate Active inventory
template: `operational_scope_1_t_co2e`,
`operational_scope_2_market_t_co2e`,
`operational_scope_2_location_t_co2e`.

## Reconciling facility-level electricity to corporate-level

Electricity is the only sector with facility-level Scope 1 in NGER.
Facility-level total should reconcile to the corporate-level
electricity total, within rounding.

```{r}
elec_fac <- cer_nger_electricity(year = 2025)

# Electricity subset at corporate level (heuristic: ANZSIC 26 or
# a "generation" substring).
elec_corp <- corp[grepl("electric|power|generat",
                         corp$reporting_corporation,
                         ignore.case = TRUE), ]

fac_total <- sum(elec_fac$emissions_t_co2e, na.rm = TRUE)
corp_total <- sum(elec_corp$scope_1_emissions_t_co2e, na.rm = TRUE)

# Gap is informative: mismatch suggests corporate/facility
# attribution differs or NGER updated one but not the other.
pct_gap <- (fac_total - corp_total) / corp_total
pct_gap
```

A gap above 5 per cent is worth investigating before reporting
aggregate electricity emissions.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nger_scope.R
\name{cer_nger_scope}
\alias{cer_nger_scope}
\title{Select NGER columns by scope with explicit errors}
\usage{
cer_nger_scope(
  x,
  scope = c("1_plus_2", "1", "2_market", "2_location"),
  col_pattern = NULL
)
}
\arguments{
\item{x}{A \code{cer_tbl} or data frame from \code{\link[=cer_nger_corporate]{cer_nger_corporate()}}.}

\item{scope}{One of \code{"1_plus_2"} (default), \code{"1"}, \code{"2_market"},
or \code{"2_location"}.}

\item{col_pattern}{Optional regex override for the target column.}
}
\value{
A data frame with an \code{emissions_t_co2e} column populated
by the requested scope and a \code{scope} column recording which
scope was selected.
}
\description{
Given an NGER corporate-level data frame from
\code{\link[=cer_nger_corporate]{cer_nger_corporate()}}, select the requested scope columns and
error if they are not present. Avoids the silent-misaggregation
failure mode where a user sums a column that is actually Scope
1+2 combined when they intended Scope 1 only.
}
\details{
Column-name detection is heuristic because NGER renames columns
year to year. Pass \code{col_pattern} to override.
}
\examples{
\donttest{
op <- options(cer.cache_dir = tempdir())
try({
  x <- cer_nger_corporate(2025)
  cer_nger_scope(x, scope = "1")
})
options(op)
}
}
\seealso{
Other nger: 
\code{\link{cer_nger_climate_active}()},
\code{\link{cer_nger_corporate}()},
\code{\link{cer_nger_electricity}()}
}
\concept{nger}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/safeguard_reform.R
\name{cer_safeguard_industry_baselines}
\alias{cer_safeguard_industry_baselines}
\title{Bundled Safeguard industry-average default baselines}
\usage{
cer_safeguard_industry_baselines()
}
\value{
A \code{cer_tbl} of industry default baselines.
}
\description{
Bundled Safeguard industry-average default baselines
}
\examples{
cer_safeguard_industry_baselines()
}
\seealso{
Other safeguard: 
\code{\link{cer_safeguard_baseline_trajectory}()},
\code{\link{cer_safeguard_facilities}()},
\code{\link{cer_safeguard_teba_facilities}()}
}
\concept{safeguard}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accu_integrity.R
\name{cer_method_integrity}
\alias{cer_method_integrity}
\title{ACCU method integrity scorecard}
\usage{
cer_method_integrity(tier = NULL, status = NULL)
}
\arguments{
\item{tier}{Optional filter on \code{integrity_tier}. One or more of
\code{"high"}, \code{"standard"}, \code{"contested"}.}

\item{status}{Optional filter on method \code{status}. One or more of
\code{"active"}, \code{"suspended"}, \code{"under_review"}, \code{"superseded"},
\code{"expired"}.}
}
\value{
A \code{cer_tbl} with one row per method (19 rows at v0.2.0).
}
\description{
Returns a method-level integrity scorecard for all ACCU methods.
Columns include \code{integrity_tier} (high / standard / contested),
\code{chubb_affected} (whether the Chubb 2022 Independent Review
flagged the method), \code{chubb_recommendation_applies} (whether a
specific Chubb recommendation targets the method), and
\code{federal_register_instrument} (authoritative legislative
instrument ID).
}
\details{
Use this to filter or weight ACCU aggregates, or to annotate a
chart with integrity context. For example, an analyst reporting
"185 million ACCUs issued to date" should, at minimum, break
the total down by \code{integrity_tier} and flag the contested share.
}
\examples{
cer_method_integrity()
cer_method_integrity(tier = "contested")
cer_method_integrity(status = "active")
}
\references{
Chubb, I. (2022). \emph{Independent Review of Australian Carbon
Credit Units}. Commonwealth of Australia, Department of
Climate Change, Energy, the Environment and Water.

Macintosh, A., Butler, D., Larraondo, P., Evans, M.C., Ansell,
D., Gibbons, P., Lindenmayer, D., Waschka, M. and Fisher, R.
(2022). "The emperor's new clothes: assessing the integrity
of ACCUs." Australian National University working paper series.

Butler, D., Macintosh, A. and Pouliot, M. (2023). "Response to
the Chubb Review." Australian National University.

Ansell, D. \emph{et al.} (2020). "Contemporary fire regimes of
northern Australia: a review of current patterns, impacts and
challenges." \emph{Environmental Research Reviews}.
}
\seealso{
Other accu: 
\code{\link{cer_accu_aggregate}()},
\code{\link{cer_accu_contracts}()},
\code{\link{cer_accu_issuances}()},
\code{\link{cer_accu_methods}()},
\code{\link{cer_accu_projects}()},
\code{\link{cer_accu_relinquishments}()},
\code{\link{cer_method_determination}()}
}
\concept{accu}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cer-package.R
\docType{package}
\name{cer-package}
\alias{cer}
\alias{cer-package}
\title{cer: Australian Clean Energy Regulator Data}
\description{
Tidy R access to data published by the Australian Clean Energy
Regulator on carbon credits, the Safeguard Mechanism, renewable
energy certificates, and greenhouse gas reporting.
}
\section{Main function families}{

\itemize{
\item \code{\link[=cer_accu_projects]{cer_accu_projects()}}, \code{\link[=cer_accu_issuances]{cer_accu_issuances()}},
\code{\link[=cer_accu_contracts]{cer_accu_contracts()}}, \code{\link[=cer_accu_methods]{cer_accu_methods()}},
\code{\link[=cer_accu_relinquishments]{cer_accu_relinquishments()}}: Australian Carbon Credit Unit
('ACCU') Scheme.
\item \code{\link[=cer_safeguard_facilities]{cer_safeguard_facilities()}}: Safeguard Mechanism covered
emissions, baselines, and SMC issuance.
\item \code{\link[=cer_nger_corporate]{cer_nger_corporate()}}, \code{\link[=cer_nger_electricity]{cer_nger_electricity()}}: National
Greenhouse and Energy Reporting ('NGER') scheme.
\item \code{\link[=cer_lgc_power_stations]{cer_lgc_power_stations()}}: Large-scale Renewable Energy Target
('LRET') accredited power stations.
\item \code{\link[=cer_sres_installations]{cer_sres_installations()}}: Small-scale Renewable Energy Scheme
('SRES') postcode installation data.
\item \code{\link[=cer_qcmr]{cer_qcmr()}}: Quarterly Carbon Market Report data workbook.
\item \code{\link[=cer_cache_info]{cer_cache_info()}}, \code{\link[=cer_clear_cache]{cer_clear_cache()}}: cache management.
}
}

\section{Data source}{

All data is published by the Clean Energy Regulator at
\url{https://cer.gov.au/markets/reports-and-data} under a Creative
Commons Attribution 4.0 International licence.
}

\seealso{
Useful links:
\itemize{
  \item \url{https://charlescoverdale.github.io/cer/}
  \item \url{https://github.com/charlescoverdale/cer}
  \item Report bugs at \url{https://github.com/charlescoverdale/cer/issues}
}

}
\author{
\strong{Maintainer}: Charles Coverdale \email{charlesfcoverdale@gmail.com}

}
\concept{ACCU}
\concept{Australian climate policy}
\concept{NGER}
\concept{Safeguard Mechanism}
\concept{carbon markets}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legislative.R
\name{cer_legislative_instrument}
\alias{cer_legislative_instrument}
\title{Build a Federal Register of Legislation URL}
\usage{
cer_legislative_instrument(instrument)
}
\arguments{
\item{instrument}{Character identifier (e.g. \code{"F2024L01292"}).
Case-insensitive.}
}
\value{
A length-1 character URL string.
}
\description{
Given a Legislative Instrument identifier (e.g. \code{"F2024L01292"}
for the Environmental Plantings 2024 method), return the stable
URL on the Federal Register of Legislation. This is the
authoritative source of truth for ACCU methods, Safeguard Rule
amendments, and NGER Determinations.
}
\examples{
cer_legislative_instrument("F2024L01292")
cer_legislative_instrument("F2015L00398")
}
\references{
Commonwealth of Australia. \emph{Legislation Act 2003} establishes
the Federal Register of Legislation as the authoritative
source for Legislative Instruments.
}
\seealso{
Other reproducibility: 
\code{\link{cer_cite}()},
\code{\link{cer_deposit_zenodo}()},
\code{\link{cer_manifest}()},
\code{\link{cer_manifest_clear}()},
\code{\link{cer_manifest_write}()},
\code{\link{cer_sha256}()},
\code{\link{cer_snapshot}()}
}
\concept{reproducibility}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reconcile.R
\name{cer_reconcile}
\alias{cer_reconcile}
\title{Reconcile an aggregate against the QCMR headline figure}
\usage{
cer_reconcile(value, quarter, measure, sum_column = NULL, warn_pct = 0.02)
}
\arguments{
\item{value}{Numeric scalar or data frame. If a data frame, pass
\code{sum_column} to pick which numeric column to sum.}

\item{quarter}{Character, e.g. \code{"2024-Q4"} or \code{"2023-24"}.}

\item{measure}{Character, matching a \code{measure} value in
\code{\link[=cer_qcmr_reference]{cer_qcmr_reference()}}. For example
\code{"accu_cumulative_issuances"},
\code{"safeguard_covered_emissions_mt"}, \code{"smc_issuances"}.}

\item{sum_column}{Column name to sum when \code{value} is a data frame.}

\item{warn_pct}{Warn if |pct_diff| > this threshold. Default 0.02.}
}
\value{
A one-row data frame: \code{measure}, \code{quarter}, \code{value},
\code{reference}, \code{diff}, \code{pct_diff}, \code{unit}, \code{source}.
}
\description{
Compares a scalar or data-frame sum against the CER Quarterly
Carbon Market Report headline for the same quarter and measure.
Warns on absolute percentage gap > 2\% (default). Small gaps
(sub-1\%) are expected because ACCU project registers are
retroactively revised between QCMR releases.
}
\examples{
cer_reconcile(value = 185e6,
              quarter = "2024-Q4",
              measure = "accu_cumulative_issuances")
}
\references{
Clean Energy Regulator (quarterly). \emph{Quarterly Carbon Market
Report}. \url{https://cer.gov.au/markets/reports-and-data/quarterly-carbon-market-reports}
}
\seealso{
Other reconciliation: 
\code{\link{cer_qcmr_reference}()}
}
\concept{reconciliation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accu_integrity.R
\name{cer_accu_aggregate}
\alias{cer_accu_aggregate}
\title{Integrity-weighted ACCU aggregation}
\usage{
cer_accu_aggregate(x, value_col = "accus_issued", warn_contested_pct = 0.05)
}
\arguments{
\item{x}{A \code{cer_tbl} or data frame with columns \code{method} and
(typically) \code{accus_issued}.}

\item{value_col}{Column to sum. Default \code{"accus_issued"}.}

\item{warn_contested_pct}{Warn if the contested tier exceeds
this share of the total. Default \code{0.05} (5\%).}
}
\value{
A data frame with one row per integrity tier and
columns \code{integrity_tier}, \code{projects}, \code{accus_issued_sum},
\code{share_pct}.
}
\description{
Takes an \code{ato_tbl} or data frame of ACCU project records (from
\code{\link[=cer_accu_projects]{cer_accu_projects()}}), joins the method integrity scorecard,
and aggregates by integrity tier. Emits a warning if contested
or under-review methods contribute materially to the total so
the analyst cannot silently aggregate them into a single
headline figure.
}
\examples{
\donttest{
op <- options(cer.cache_dir = tempdir())
try({
  p <- cer_accu_projects()
  cer_accu_aggregate(p)
})
options(op)
}
}
\seealso{
Other accu: 
\code{\link{cer_accu_contracts}()},
\code{\link{cer_accu_issuances}()},
\code{\link{cer_accu_methods}()},
\code{\link{cer_accu_projects}()},
\code{\link{cer_accu_relinquishments}()},
\code{\link{cer_method_determination}()},
\code{\link{cer_method_integrity}()}
}
\concept{accu}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/manifest.R
\name{cer_manifest_clear}
\alias{cer_manifest_clear}
\title{Clear the session manifest}
\usage{
cer_manifest_clear()
}
\value{
Invisibly \code{NULL}.
}
\description{
Clear the session manifest
}
\examples{
cer_manifest_clear()
}
\seealso{
Other reproducibility: 
\code{\link{cer_cite}()},
\code{\link{cer_deposit_zenodo}()},
\code{\link{cer_legislative_instrument}()},
\code{\link{cer_manifest}()},
\code{\link{cer_manifest_write}()},
\code{\link{cer_sha256}()},
\code{\link{cer_snapshot}()}
}
\concept{reproducibility}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nger_scope.R
\name{cer_nger_climate_active}
\alias{cer_nger_climate_active}
\title{Translate NGER scope columns to Climate Active categories}
\usage{
cer_nger_climate_active(x)
}
\arguments{
\item{x}{A \code{cer_tbl} or data frame from \code{\link[=cer_nger_corporate]{cer_nger_corporate()}}.}
}
\value{
A data frame with Climate Active-style columns:
\code{operational_scope_1_t_co2e}, \code{operational_scope_2_location_t_co2e},
\code{operational_scope_2_market_t_co2e}.
}
\description{
Climate Active (Australia's voluntary carbon-neutral certification)
uses a Scope 1 / Scope 2 (market-based and location-based) /
Scope 3 taxonomy. NGER publishes Scope 1 and Scope 2 only (Scope 3
is not NGER-reportable). This helper renames columns so an NGER
output flows into a Climate Active inventory template.
}
\examples{
\donttest{
op <- options(cer.cache_dir = tempdir())
try({
  x <- cer_nger_corporate(2025)
  cer_nger_climate_active(x)
})
options(op)
}
}
\references{
Department of Climate Change, Energy, the Environment and Water
(annual). \emph{Climate Active Carbon Neutral Standard for
Organisations}. Commonwealth of Australia.

Greenhouse Gas Protocol (2004). \emph{A Corporate Accounting and
Reporting Standard (Revised Edition)}. World Resources Institute.
}
\seealso{
Other nger: 
\code{\link{cer_nger_corporate}()},
\code{\link{cer_nger_electricity}()},
\code{\link{cer_nger_scope}()}
}
\concept{nger}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lret_sres.R
\name{cer_lgc_power_stations}
\alias{cer_lgc_power_stations}
\title{LRET accredited power stations}
\source{
Clean Energy Regulator, Historical large-scale renewable
energy supply data:
\url{https://cer.gov.au/markets/reports-and-data/large-scale-renewable-energy-data/historical-large-scale-renewable-energy-supply-data}.
Licensed under CC BY 4.0.
}
\usage{
cer_lgc_power_stations(technology = NULL, state = NULL)
}
\arguments{
\item{technology}{Optional character vector of technology
substrings (e.g. \code{"solar"}, \code{"wind"}, \code{"hydro"}, \code{"bioenergy"}).}

\item{state}{Optional character vector of Australian state codes
(e.g. \code{c("VIC", "SA")}).}
}
\value{
A \code{cer_tbl} with one row per accredited power station,
including accreditation date, technology, capacity (MW), and
state.
}
\description{
Power stations accredited under the Large-scale Renewable
Energy Target ('LRET') and eligible to create Large-scale
Generation Certificates ('LGCs').
}
\examples{
\donttest{
op <- options(cer.cache_dir = tempdir())
s <- cer_lgc_power_stations(technology = "solar", state = "VIC")
head(s)
options(op)
}
}
\references{
Commonwealth of Australia. \emph{Renewable Energy (Electricity)
Act 2000}. Establishes the Renewable Energy Target (RET), the
LGC (Large-scale Generation Certificate) and the shortfall
charge mechanism.

Clean Energy Regulator (annual). \emph{LRET: Accreditation and
eligibility guidelines for power stations}.

Australian Energy Market Commission (2023). \emph{Integrated
System Plan 2024} reference on renewable capacity expansion.
}
\concept{lret}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{cer_clear_cache}
\alias{cer_clear_cache}
\title{Clear the cer cache}
\usage{
cer_clear_cache()
}
\value{
Invisibly returns \code{NULL}.
}
\description{
Deletes all locally cached files. The next call to any data
function will re-download from the Clean Energy Regulator.
}
\examples{
\donttest{
op <- options(cer.cache_dir = tempdir())
cer_clear_cache()
options(op)
}
}
\seealso{
Other configuration: 
\code{\link{cer_cache_info}()}
}
\concept{configuration}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/safeguard_reform.R
\name{cer_safeguard_baseline_trajectory}
\alias{cer_safeguard_baseline_trajectory}
\title{Compute a declining Safeguard baseline trajectory}
\usage{
cer_safeguard_baseline_trajectory(industry, from_year = 2023, to_year = 2030)
}
\arguments{
\item{industry}{Character scalar matching an \code{industry} value
from \code{\link[=cer_safeguard_industry_baselines]{cer_safeguard_industry_baselines()}}.}

\item{from_year}{Start year (default 2023 for FY 2023-24).}

\item{to_year}{End year (default 2029 for FY 2029-30).}
}
\value{
A data frame with \code{year}, \code{financial_year}, \code{baseline},
and \code{decline_factor} columns.
}
\description{
Implements the 4.9\% per-annum nominal decline in industry-average
default baselines introduced by the 2023 reform. The formula is:
}
\details{
\deqn{B_t = B_{2023\text{-}24} \times (1 - 0.049)^{t - 2023}}

where \code{t} is the starting calendar year of the financial year.
Default values are drawn from
\code{inst/extdata/safeguard_industry_baselines.csv}.
}
\examples{
cer_safeguard_baseline_trajectory("Aluminium smelting")
cer_safeguard_baseline_trajectory("Cement clinker",
                                   from_year = 2023, to_year = 2030)
}
\references{
Commonwealth of Australia (2023). \emph{Safeguard Mechanism
(Crediting) Amendment Act 2023}. Introduces 4.9\% p.a. nominal
baseline decline to 2030.

Clean Energy Regulator (annual). \emph{Safeguard Mechanism:
declining baselines}. Methodology notes accompanying annual
baseline determinations.
}
\seealso{
Other safeguard: 
\code{\link{cer_safeguard_facilities}()},
\code{\link{cer_safeguard_industry_baselines}()},
\code{\link{cer_safeguard_teba_facilities}()}
}
\concept{safeguard}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/safeguard.R
\name{cer_nger_corporate}
\alias{cer_nger_corporate}
\title{NGER corporate emissions and energy data}
\source{
Clean Energy Regulator, NGER reporting data and registers:
\url{https://cer.gov.au/markets/reports-and-data/nger-reporting-data-and-registers}.
Licensed under CC BY 4.0.
}
\usage{
cer_nger_corporate(year = 2025)
}
\arguments{
\item{year}{Integer. Reporting year ending 30 June.}
}
\value{
A \code{cer_tbl} with one row per reporting corporation.
}
\description{
Controlling-corporation-level Scope 1 and Scope 2 emissions and
energy use from the National Greenhouse and Energy Reporting
('NGER') scheme.
}
\details{
Published annually by the CER on 28 February. Facility-level
Scope 1 and 2 data is only published for the electricity
sector; non-electricity sectors report at this
controlling-corporation level only.
}
\examples{
\donttest{
op <- options(cer.cache_dir = tempdir())
try({
  x <- cer_nger_corporate(year = 2025)
  head(x)
})
options(op)
}
}
\references{
Commonwealth of Australia. \emph{National Greenhouse and Energy
Reporting Act 2007}; \emph{National Greenhouse and Energy
Reporting (Measurement) Determination 2008} (as amended
annually).

Greenhouse Gas Protocol (2004). \emph{A Corporate Accounting
and Reporting Standard (Revised Edition)}. World Resources
Institute.

Department of Climate Change, Energy, the Environment and Water
(annual). \emph{NGER Technical Guidelines}. Methodology for
Scope 1 and Scope 2 calculations.
}
\seealso{
Other nger: 
\code{\link{cer_nger_climate_active}()},
\code{\link{cer_nger_electricity}()},
\code{\link{cer_nger_scope}()}
}
\concept{nger}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accu.R
\name{cer_accu_methods}
\alias{cer_accu_methods}
\title{Approved ACCU Scheme methods}
\source{
Clean Energy Regulator ACCU Scheme methods index
\url{https://cer.gov.au/schemes/australian-carbon-credit-unit-scheme/accu-scheme-methods}.
}
\usage{
cer_accu_methods()
}
\value{
A \code{cer_tbl} with columns \code{method}, \code{sector},
\code{short_name}, \code{status}, \code{commenced}, and \code{source_url}.
}
\description{
A static table of ACCU Scheme methods grouped by sector
(Agriculture, Energy Efficiency, Landfill and Waste,
Mining/Oil/Gas, Vegetation, Savanna, Blue Carbon,
Transport). Built from the CER methods index at build time.
}
\details{
Because the CER publishes methods as per-method PDFs on the
Federal Register of Legislation rather than a machine-readable
index, this function returns a package-embedded lookup. The
\code{status} column captures post-Chubb Review changes: methods
are flagged as \code{"active"}, \code{"expired"}, \code{"suspended"}, or
\code{"under_review"}. Run \code{\link[=cer_accu_projects]{cer_accu_projects()}} then
\code{unique(x$method)} to cross-check against currently-registered
projects.
}
\examples{
m <- cer_accu_methods()
head(m)
}
\seealso{
Other accu: 
\code{\link{cer_accu_aggregate}()},
\code{\link{cer_accu_contracts}()},
\code{\link{cer_accu_issuances}()},
\code{\link{cer_accu_projects}()},
\code{\link{cer_accu_relinquishments}()},
\code{\link{cer_method_determination}()},
\code{\link{cer_method_integrity}()}
}
\concept{accu}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accu.R
\name{cer_accu_contracts}
\alias{cer_accu_contracts}
\title{ACCU Carbon Abatement Contract register}
\source{
Clean Energy Regulator Carbon Abatement Contract register
(CC BY 4.0).
}
\usage{
cer_accu_contracts(auction = NULL)
}
\arguments{
\item{auction}{Optional character vector of auction identifiers
(e.g. \code{"ERF Auction 18"}).}
}
\value{
A \code{cer_tbl} with one row per contract.
}
\description{
Government purchase contracts under Emissions Reduction Fund
auctions administered by the CER.
}
\examples{
\donttest{
op <- options(cer.cache_dir = tempdir())
cer_accu_contracts()
options(op)
}
}
\seealso{
Other accu: 
\code{\link{cer_accu_aggregate}()},
\code{\link{cer_accu_issuances}()},
\code{\link{cer_accu_methods}()},
\code{\link{cer_accu_projects}()},
\code{\link{cer_accu_relinquishments}()},
\code{\link{cer_method_determination}()},
\code{\link{cer_method_integrity}()}
}
\concept{accu}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zenodo.R
\name{cer_deposit_zenodo}
\alias{cer_deposit_zenodo}
\title{Prepare a Zenodo deposit payload for the session manifest}
\usage{
cer_deposit_zenodo(
  title = NULL,
  description = NULL,
  creators = list(list(name = "Anonymous")),
  keywords = c("Clean Energy Regulator", "carbon markets", "Australia", "ACCU",
    "Safeguard", "reproducibility"),
  upload = FALSE,
  sandbox = FALSE,
  token = Sys.getenv("ZENODO_TOKEN")
)
}
\arguments{
\item{title}{Deposit title. Defaults to a snapshot-date string.}

\item{description}{Free-text description. Defaults to a short
auto-generated summary of datasets fetched.}

\item{creators}{List of creator records.}

\item{keywords}{Character vector of keywords.}

\item{upload}{Logical. If \code{TRUE}, POSTs to Zenodo.}

\item{sandbox}{Logical. If \code{TRUE}, uses Zenodo Sandbox for tests.}

\item{token}{Zenodo personal access token. Defaults to
\code{Sys.getenv("ZENODO_TOKEN")}.}
}
\value{
A list with \code{payload}, \code{manifest_path}, and (on upload)
\code{deposit_id}, \code{doi_prereserve}, \code{url}.
}
\description{
Builds Zenodo-shaped metadata for a data deposit using the
current \code{\link[=cer_manifest]{cer_manifest()}} and the snapshot pin set via
\code{\link[=cer_snapshot]{cer_snapshot()}}. Dry run by default; pass \code{upload = TRUE} and
a token to actually deposit.
}
\details{
Because CER data is revised retroactively, Zenodo DOIs are the
strongest reproducibility artefact available: they freeze the
exact bytes of the ACCU project register, Safeguard facility
file, or QCMR workbook at the date of analysis.
}
\examples{
\donttest{
cer_snapshot("2026-04-24")
cer_deposit_zenodo(
  title = "CER data snapshot for working paper v1",
  creators = list(list(name = "Author, A.")),
  upload = FALSE
)
}
}
\seealso{
Other reproducibility: 
\code{\link{cer_cite}()},
\code{\link{cer_legislative_instrument}()},
\code{\link{cer_manifest}()},
\code{\link{cer_manifest_clear}()},
\code{\link{cer_manifest_write}()},
\code{\link{cer_sha256}()},
\code{\link{cer_snapshot}()}
}
\concept{reproducibility}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qcmr.R
\name{cer_qcmr}
\alias{cer_qcmr}
\title{Quarterly Carbon Market Report data workbook}
\source{
Clean Energy Regulator, Quarterly Carbon Market Reports:
\url{https://cer.gov.au/markets/reports-and-data/quarterly-carbon-market-reports}.
Licensed under CC BY 4.0.
}
\usage{
cer_qcmr(quarter = "latest", sheet = "Contents")
}
\arguments{
\item{quarter}{Character. Quarter identifier in \code{"YYYYqN"} form
(e.g. \code{"2025q4"} for the December 2025 quarter). Defaults to
\code{"latest"}, which resolves to the most recent release.}

\item{sheet}{Sheet name or integer index. The CER QCMR workbook
contains a \code{Contents} sheet (default) plus one sheet per
figure (e.g. \code{"Figure 1.1"}, \code{"Figure 3.14"}). Pass a figure
name to extract that figure's data.}
}
\value{
A \code{cer_tbl} with the requested sheet's contents.
}
\description{
Aggregate carbon market statistics from the CER's Quarterly
Carbon Market Report, including ACCU issuances, LGC creations,
STC volumes, unit surrenders, and Safeguard Mechanism
developments.
}
\examples{
\donttest{
op <- options(cer.cache_dir = tempdir())
try({
  # Contents sheet lists all available figures
  toc <- cer_qcmr("latest")
  head(toc)
  # Fetch a specific figure
  fig <- cer_qcmr("latest", sheet = "Figure 1.1")
})
options(op)
}
}
\references{
Clean Energy Regulator (quarterly). \emph{Quarterly Carbon
Market Report}. Methodology notes accompany each release.

Clean Energy Regulator (2024). \emph{Carbon Market Indicators:
methodology and data dictionary}.
}
\concept{qcmr}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interop.R
\name{cer_international_comparator}
\alias{cer_international_comparator}
\title{Bundled international carbon market comparator}
\usage{
cer_international_comparator()
}
\value{
A \code{cer_tbl} of indicative comparator data.
}
\description{
Returns headline price/volume figures for comparison between
ACCU (Australia) and major international markets: EU ETS, New
Zealand ETS, California-Quebec (WCI), Verra VCUs, and Gold
Standard. Values are indicative snapshots. For live data use
the companion \code{carbondata} package.
}
\examples{
cer_international_comparator()
}
\references{
International Carbon Action Partnership (annual). \emph{Emissions
Trading Worldwide: Status Report}. \url{https://icapcarbonaction.com/en}

World Bank (annual). \emph{State and Trends of Carbon Pricing}.
}
\seealso{
Other interop: 
\code{\link{cer_to_carbondata}()}
}
\concept{interop}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/safeguard.R
\name{cer_safeguard_facilities}
\alias{cer_safeguard_facilities}
\title{Safeguard Mechanism facility-level data}
\source{
Clean Energy Regulator, Safeguard data:
\url{https://cer.gov.au/markets/reports-and-data/safeguard-data}.
Licensed under CC BY 4.0.
}
\usage{
cer_safeguard_facilities(year = 2025)
}
\arguments{
\item{year}{Integer. Reporting year ending 30 June (e.g. \code{2025}
for 2024-25). Defaults to the most recently released year.}
}
\value{
A \code{cer_tbl} with one row per facility.
}
\description{
Covered emissions, baselines, Safeguard Mechanism Credits (SMCs)
issued or surrendered, and abatement information for facilities
above the 100,000 tonne CO2e threshold.
}
\details{
Published annually by the CER on 15 April following each
reporting year. The download URL is resolved dynamically from
the CER's Safeguard landing page, so no slug changes here are
required when new years publish.
}
\examples{
\donttest{
op <- options(cer.cache_dir = tempdir())
try({
  f <- cer_safeguard_facilities(year = 2025)
  head(f)
})
options(op)
}
}
\references{
Commonwealth of Australia. \emph{National Greenhouse and Energy
Reporting Act 2007}.

Commonwealth of Australia. \emph{National Greenhouse and Energy
Reporting (Safeguard Mechanism) Rule 2015} (as amended).

Commonwealth of Australia (2023). \emph{Safeguard Mechanism
(Crediting) Amendment Act 2023}. Introduced declining industry
baselines (4.9\% p.a. nominal) and Safeguard Mechanism Credits
(SMCs).

Climate Change Authority (2023). \emph{Review of the Safeguard
Mechanism}.
}
\seealso{
Other safeguard: 
\code{\link{cer_safeguard_baseline_trajectory}()},
\code{\link{cer_safeguard_industry_baselines}()},
\code{\link{cer_safeguard_teba_facilities}()}
}
\concept{safeguard}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accu.R
\name{cer_accu_issuances}
\alias{cer_accu_issuances}
\title{ACCU issuances by project}
\source{
Clean Energy Regulator ACCU project register (CC BY 4.0).
}
\usage{
cer_accu_issuances(method = NULL, project_id = NULL, from = NULL, to = NULL)
}
\arguments{
\item{method}{Optional character vector of method substrings.}

\item{project_id}{Optional character vector of ERF / EOP project
identifiers.}

\item{from, to}{Optional \code{Date} (or YYYY-MM-DD string) for
crediting-period overlap filter. \code{NULL} returns all projects.}
}
\value{
A \code{cer_tbl} with one row per project.
}
\description{
Derived from the ACCU Scheme project register: returns one row
per project with \code{accus_issued}, project metadata, and total
issuance net of relinquishments. For method- or period-level
issuance tracking, query \code{\link[=cer_accu_projects]{cer_accu_projects()}} directly.
}
\examples{
\donttest{
op <- options(cer.cache_dir = tempdir())
i <- cer_accu_issuances(method = "savanna")
head(i)
options(op)
}
}
\seealso{
Other accu: 
\code{\link{cer_accu_aggregate}()},
\code{\link{cer_accu_contracts}()},
\code{\link{cer_accu_methods}()},
\code{\link{cer_accu_projects}()},
\code{\link{cer_accu_relinquishments}()},
\code{\link{cer_method_determination}()},
\code{\link{cer_method_integrity}()}
}
\concept{accu}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/safeguard.R
\name{cer_nger_electricity}
\alias{cer_nger_electricity}
\title{NGER electricity sector facility emissions}
\source{
Clean Energy Regulator, National Greenhouse and Energy
Register:
\url{https://cer.gov.au/markets/reports-and-data/nger-reporting-data-and-registers}.
Licensed under CC BY 4.0.
}
\usage{
cer_nger_electricity(year = 2025, facility = NULL)
}
\arguments{
\item{year}{Integer. Reporting year ending 30 June.}

\item{facility}{Optional character vector of facility name
substrings (case-insensitive).}
}
\value{
A \code{cer_tbl} with one row per generator facility.
}
\description{
Facility-level Scope 1 emissions and generated electricity for
facilities in the electricity generation sector that met the
NGER reporting threshold.
}
\examples{
\donttest{
op <- options(cer.cache_dir = tempdir())
try({
  e <- cer_nger_electricity(year = 2025, facility = "Bayswater")
  head(e)
})
options(op)
}
}
\seealso{
Other nger: 
\code{\link{cer_nger_climate_active}()},
\code{\link{cer_nger_corporate}()},
\code{\link{cer_nger_scope}()}
}
\concept{nger}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/manifest.R
\name{cer_manifest_write}
\alias{cer_manifest_write}
\title{Write the session manifest to a file}
\usage{
cer_manifest_write(path, format = c("auto", "csv", "yaml", "json"))
}
\arguments{
\item{path}{Output file path. Extension determines format when
\code{format = "auto"}.}

\item{format}{One of \code{"auto"}, \code{"csv"}, \code{"yaml"}, \code{"json"}.}
}
\value{
Invisibly, the absolute path written.
}
\description{
Write the session manifest to a file
}
\examples{
\donttest{
p <- tempfile(fileext = ".csv")
cer_manifest_clear()
cer_manifest_write(p)
}
}
\seealso{
Other reproducibility: 
\code{\link{cer_cite}()},
\code{\link{cer_deposit_zenodo}()},
\code{\link{cer_legislative_instrument}()},
\code{\link{cer_manifest}()},
\code{\link{cer_manifest_clear}()},
\code{\link{cer_sha256}()},
\code{\link{cer_snapshot}()}
}
\concept{reproducibility}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cite.R
\name{cer_cite}
\alias{cer_cite}
\title{Cite a cer_tbl or URL in BibTeX, plain-text, or APA form}
\usage{
cer_cite(x, style = c("text", "bibtex", "apa"), doi = NULL)
}
\arguments{
\item{x}{A \code{cer_tbl} (as returned by any \verb{cer_*} data function)
or a character URL.}

\item{style}{One of \code{"text"} (default), \code{"bibtex"}, or \code{"apa"}.}

\item{doi}{Optional DOI (e.g. from \code{\link[=cer_deposit_zenodo]{cer_deposit_zenodo()}}) to
include in BibTeX output as a \code{doi} field and APA suffix.}
}
\value{
A character string.
}
\description{
Returns a citation suitable for footnotes, papers, and carbon
market analysis reports. Uses the provenance attributes attached
to every \code{cer_tbl}: source URL, licence, retrieval date, title,
snapshot pin (from \code{\link[=cer_snapshot]{cer_snapshot()}}), and SHA-256 digest.
}
\details{
For carbon market research, the BibTeX \code{note} field includes
the snapshot date and first 12 hex characters of the SHA-256,
which are the minimum provenance fields needed to defend a
published figure against a reviewer.
}
\examples{
x <- data.frame(a = 1)
x <- structure(x,
  cer_source = "https://cer.gov.au/markets/reports-and-data/accu-project-and-contract-register",
  cer_licence = "CC BY 4.0",
  cer_retrieved = as.POSIXct("2026-04-24 00:00:00", tz = "UTC"),
  cer_title = "ACCU project register",
  cer_sha256 = "abc123def456",
  cer_snapshot_date = "2026-04-24",
  class = c("cer_tbl", "data.frame"))
cer_cite(x)
cer_cite(x, style = "bibtex")
# DOI style: supply any minted DOI (Zenodo, DataCite, etc.).
# The placeholder below is illustrative only.
cer_cite(x, style = "apa", doi = "10.5281/zenodo.XXXXXXXX")
}
\seealso{
Other reproducibility: 
\code{\link{cer_deposit_zenodo}()},
\code{\link{cer_legislative_instrument}()},
\code{\link{cer_manifest}()},
\code{\link{cer_manifest_clear}()},
\code{\link{cer_manifest_write}()},
\code{\link{cer_sha256}()},
\code{\link{cer_snapshot}()}
}
\concept{reproducibility}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lret_sres.R
\name{cer_sres_installations}
\alias{cer_sres_installations}
\title{SRES small-scale installations by postcode}
\source{
Clean Energy Regulator, Small-scale installation
postcode data:
\url{https://cer.gov.au/markets/reports-and-data/small-scale-installation-postcode-data}.
Licensed under CC BY 4.0.
}
\usage{
cer_sres_installations(
  technology = c("solar_pv", "swh", "heat_pump", "battery"),
  postcode = NULL,
  measure = c("installations", "capacity")
)
}
\arguments{
\item{technology}{Character. One of \code{"solar_pv"} (default), \code{"swh"}
(solar water heater), \code{"heat_pump"}, or \code{"battery"}.}

\item{postcode}{Optional character vector of 4-digit Australian
postcodes to filter on.}

\item{measure}{Character. \code{"installations"} (default, counts) or
\code{"capacity"} (total kW installed).}
}
\value{
A \code{cer_tbl}. For \code{measure = "installations"} columns
include postcode and per-month install counts. For
\code{measure = "capacity"}, per-month kW capacity.
}
\description{
Monthly Small-scale Renewable Energy Scheme ('SRES') installation
counts or capacity by postcode. Covers rooftop solar PV, solar
water heaters, air-source heat pumps, and small batteries.
}
\examples{
\donttest{
op <- options(cer.cache_dir = tempdir())
inst <- cer_sres_installations(postcode = c("2000", "3000"))
head(inst)
options(op)
}
}
\references{
Commonwealth of Australia. \emph{Renewable Energy (Electricity)
Act 2000}, Small-scale Renewable Energy Scheme provisions.

Australian Energy Market Operator (2024). \emph{Distributed PV
Forecasting Methodology}. AEMO uses SRES installation data
as input to ISP and ESOO forecasts.

Best, R., Burke, P.J. and Nishitateno, S. (2019).
"Understanding the determinants of rooftop solar
installation: evidence from household surveys in Australia."
\emph{Australian Journal of Agricultural and Resource
Economics}, 63(4), 922-939.
\doi{10.1111/1467-8489.12319}
}
\concept{sres}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reconcile.R
\name{cer_qcmr_reference}
\alias{cer_qcmr_reference}
\title{QCMR headline reference totals}
\usage{
cer_qcmr_reference(quarter = NULL, measure = NULL)
}
\arguments{
\item{quarter}{Optional character filter (e.g. \code{"2024-Q4"}).}

\item{measure}{Optional character filter (e.g.
\code{"accu_cumulative_issuances"}).}
}
\value{
A \code{cer_tbl} of reference totals.
}
\description{
Bundled quarterly Clean Energy Regulator QCMR headline figures
for cross-validation of live-fetched aggregates.
}
\examples{
cer_qcmr_reference()
cer_qcmr_reference(quarter = "2024-Q4")
}
\seealso{
Other reconciliation: 
\code{\link{cer_reconcile}()}
}
\concept{reconciliation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{cer_cache_info}
\alias{cer_cache_info}
\title{Inspect the local cer cache}
\usage{
cer_cache_info()
}
\value{
A list with elements \code{dir}, \code{n_files}, \code{size_bytes},
\code{size_human}, and \code{files} (a data frame with \code{name},
\code{size_bytes}, \code{modified}).
}
\description{
Returns the cache directory, number of files, total size, and
a per-file summary. Useful for deciding whether to call
\code{\link[=cer_clear_cache]{cer_clear_cache()}}.
}
\examples{
\donttest{
op <- options(cer.cache_dir = tempdir())
cer_cache_info()
options(op)
}
}
\seealso{
Other configuration: 
\code{\link{cer_clear_cache}()}
}
\concept{configuration}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cer_tbl.R
\name{print.cer_tbl}
\alias{print.cer_tbl}
\title{Print a cer_tbl}
\usage{
\method{print}{cer_tbl}(x, ...)
}
\arguments{
\item{x}{A \code{cer_tbl} object.}

\item{...}{Passed to the next method.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Prints a provenance header (title, source URL, licence,
retrieval time, dimensions) followed by the data frame.
}
\examples{
x <- data.frame(project_id = "ERF123", accus_issued = 4200)
x <- structure(x, cer_title = "Demo", cer_source = "https://cer.gov.au",
               cer_licence = "CC BY 4.0", cer_retrieved = Sys.time(),
               class = c("cer_tbl", "data.frame"))
print(x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accu.R
\name{cer_accu_projects}
\alias{cer_accu_projects}
\title{ACCU Scheme project register}
\source{
Clean Energy Regulator, ACCU project and contract register:
\url{https://cer.gov.au/markets/reports-and-data/accu-project-and-contract-register}.
Licensed under CC BY 4.0.
}
\usage{
cer_accu_projects(
  state = NULL,
  method = NULL,
  status = c("active", "revoked", "completed", "all")
)
}
\arguments{
\item{state}{Optional character vector of Australian state
codes (e.g. \code{"NSW"}, \code{c("VIC", "QLD")}) to filter on. \code{NULL}
returns projects across all states.}

\item{method}{Optional character vector of method name
substrings (case-insensitive, e.g. \code{"savanna"}, \code{"landfill"},
\code{"vegetation"}).}

\item{status}{Character. One of \code{"active"} (default), \code{"revoked"},
\code{"completed"}, or \code{"all"}.}
}
\value{
A \code{cer_tbl} (data frame) with one row per ACCU project.
}
\description{
Download and parse the Clean Energy Regulator's Australian
Carbon Credit Unit ('ACCU') Scheme project register. Returns one
row per project with proponent, method, state, crediting period,
permanence period, and cumulative ACCUs issued and relinquished.
}
\details{
The register is updated monthly by the CER, usually around the
third week. Downloaded once per R session (or invalidate with
\code{\link[=cer_clear_cache]{cer_clear_cache()}}).
}
\examples{
\donttest{
op <- options(cer.cache_dir = tempdir())
p <- cer_accu_projects(state = "NSW")
head(p)
options(op)
}
}
\references{
Commonwealth of Australia. \emph{Carbon Credits (Carbon Farming
Initiative) Act 2011}. Enabling legislation for the ACCU Scheme.

Chubb, I. (2022). \emph{Independent Review of Australian Carbon
Credit Units}. Commonwealth of Australia, Department of Climate
Change, Energy, the Environment and Water.

Climate Change Authority (2024). \emph{2026 Review of the ACCU
Scheme: Issues paper}.

Macintosh, A., Butler, D., Larraondo, P., Evans, M.C., Ansell, D.,
Gibbons, P., Lindenmayer, D., Waschka, M. and Fisher, R.
(2022). "The emperor's new clothes: assessing the integrity
of ACCUs." Australian National University working paper series.
}
\seealso{
Other accu: 
\code{\link{cer_accu_aggregate}()},
\code{\link{cer_accu_contracts}()},
\code{\link{cer_accu_issuances}()},
\code{\link{cer_accu_methods}()},
\code{\link{cer_accu_relinquishments}()},
\code{\link{cer_method_determination}()},
\code{\link{cer_method_integrity}()}
}
\concept{accu}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/snapshot.R
\name{cer_snapshot}
\alias{cer_snapshot}
\title{Pin or inspect the session snapshot date}
\usage{
cer_snapshot(date)
}
\arguments{
\item{date}{ISO \code{"YYYY-MM-DD"} character, \code{Date}, or \code{POSIXct}.
Pass \code{NULL} to clear.}
}
\value{
Invisibly, the new pinned date (as \code{Date}), or \code{NULL}.
}
\description{
Call once at the top of an analysis script to declare the vintage
of CER data you intend to use. Every subsequent \verb{cer_*} fetch
records this date in the \code{cer_tbl} provenance header, in
\code{\link[=cer_manifest]{cer_manifest()}} entries, and in \code{\link[=cer_cite]{cer_cite()}} output.
}
\details{
This matters more for CER data than for most government data:
projects relinquish ACCUs retroactively, the CER transfers
projects between proponents, methods are reclassified (e.g.
post-Chubb-Review status changes), and the Quarterly Carbon
Market Report occasionally restates prior quarters. A snapshot
pin is the minimum evidence a reviewer needs to verify a
published carbon-market claim.
}
\examples{
cer_snapshot("2026-04-24")
cer_snapshot()
cer_snapshot(NULL)
}
\seealso{
Other reproducibility: 
\code{\link{cer_cite}()},
\code{\link{cer_deposit_zenodo}()},
\code{\link{cer_legislative_instrument}()},
\code{\link{cer_manifest}()},
\code{\link{cer_manifest_clear}()},
\code{\link{cer_manifest_write}()},
\code{\link{cer_sha256}()}
}
\concept{reproducibility}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integrity.R
\name{cer_sha256}
\alias{cer_sha256}
\title{Compute the SHA-256 digest of a file}
\usage{
cer_sha256(file)
}
\arguments{
\item{file}{Path to a local file.}
}
\value{
A length-1 character string (hex digest), or \code{NA} if
the file does not exist. Uses \code{digest::digest()} when the
\code{digest} package is available (recommended for research work),
falling back to \code{tools::md5sum()} otherwise.
}
\description{
Compute the SHA-256 digest of a file
}
\examples{
f <- tempfile()
writeLines("hello", f)
cer_sha256(f)
}
\seealso{
Other reproducibility: 
\code{\link{cer_cite}()},
\code{\link{cer_deposit_zenodo}()},
\code{\link{cer_legislative_instrument}()},
\code{\link{cer_manifest}()},
\code{\link{cer_manifest_clear}()},
\code{\link{cer_manifest_write}()},
\code{\link{cer_snapshot}()}
}
\concept{reproducibility}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interop.R
\name{cer_to_carbondata}
\alias{cer_to_carbondata}
\title{Remap an ACCU project data frame to the carbondata voluntary-
project schema}
\usage{
cer_to_carbondata(x)
}
\arguments{
\item{x}{A \code{cer_tbl} from \code{\link[=cer_accu_projects]{cer_accu_projects()}}.}
}
\value{
A data frame with carbondata-style columns:
\code{project_id}, \code{project_name}, \code{methodology}, \code{project_type},
\code{country}, \code{registry}, \code{issued_credits_to_date},
\code{retired_credits_to_date}.
}
\description{
The \code{carbondata} R package exposes Verra, Gold Standard, ACR,
and CAR voluntary projects under a unified schema. This function
renames ACCU register columns to match, so an analyst can
compare ACCU issuance patterns against international voluntary
markets on a consistent basis.
}
\details{
Schema mapping is lossy: ACCU-specific fields (Chubb flag,
permanence period, crediting-period dates) pass through with
original names.
}
\examples{
\donttest{
op <- options(cer.cache_dir = tempdir())
try({
  ccy <- cer_accu_projects()
  carb <- cer_to_carbondata(ccy)
  head(carb)
})
options(op)
}
}
\seealso{
Other interop: 
\code{\link{cer_international_comparator}()}
}
\concept{interop}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accu_integrity.R
\name{cer_method_determination}
\alias{cer_method_determination}
\title{Federal Register URL for an ACCU method determination}
\usage{
cer_method_determination(method_short)
}
\arguments{
\item{method_short}{Character scalar matching a \code{method_short}
value from \code{\link[=cer_method_integrity]{cer_method_integrity()}}.}
}
\value{
A length-1 character URL, or \code{NA_character_} if not found.
}
\description{
Given a method short name (e.g. \code{"HIR"}, \code{"Savanna_2026"}),
return the authoritative legislative instrument URL on the
Federal Register of Legislation. This is the technical
determination document that defines measurement, monitoring,
baseline, and permanence rules for the method.
}
\examples{
cer_method_determination("HIR")
cer_method_determination("Savanna_2026")
}
\seealso{
Other accu: 
\code{\link{cer_accu_aggregate}()},
\code{\link{cer_accu_contracts}()},
\code{\link{cer_accu_issuances}()},
\code{\link{cer_accu_methods}()},
\code{\link{cer_accu_projects}()},
\code{\link{cer_accu_relinquishments}()},
\code{\link{cer_method_integrity}()}
}
\concept{accu}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/safeguard_reform.R
\name{cer_safeguard_teba_facilities}
\alias{cer_safeguard_teba_facilities}
\title{Trade-exposed baseline-adjusted (TEBA) facility list}
\usage{
cer_safeguard_teba_facilities()
}
\value{
A \code{cer_tbl} of TEBA facilities.
}
\description{
Returns the bundled list of facilities declared as TEBA under
the 2023 reform. TEBA facilities are eligible for softer
baseline treatment reflecting international competitiveness.
}
\examples{
cer_safeguard_teba_facilities()
}
\references{
Commonwealth of Australia (2023). \emph{National Greenhouse
and Energy Reporting (Safeguard Mechanism) Rule 2015} as
amended by the Safeguard Mechanism (Crediting) Amendment
Rule 2023. TEBA definition and declaration process.
}
\seealso{
Other safeguard: 
\code{\link{cer_safeguard_baseline_trajectory}()},
\code{\link{cer_safeguard_facilities}()},
\code{\link{cer_safeguard_industry_baselines}()}
}
\concept{safeguard}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/manifest.R
\name{cer_manifest}
\alias{cer_manifest}
\title{Return the session manifest of fetched CER datasets}
\usage{
cer_manifest(format = c("df", "yaml", "json"))
}
\arguments{
\item{format}{One of \code{"df"} (default), \code{"yaml"}, or \code{"json"}.}
}
\value{
A data frame, YAML string, or JSON string.
}
\description{
Every call to a data function appends one row to the session
manifest, recording URL, dataset title, SHA-256 of the cached
file, size, retrieval timestamp, and the snapshot pin set via
\code{\link[=cer_snapshot]{cer_snapshot()}}. Duplicate URLs within a session are
deduplicated (last fetch wins).
}
\details{
For carbon market research this is the minimum artefact needed
for reproducibility: ACCU project registers and Safeguard
releases change retroactively, so a citation that points to a
URL alone is not enough. The manifest plus SHA-256 fixes the
exact bytes analysed.
}
\examples{
\donttest{
op <- options(cer.cache_dir = tempdir())
cer_manifest_clear()
cer_snapshot("2026-04-24")
cer_manifest()
options(op)
}
}
\seealso{
Other reproducibility: 
\code{\link{cer_cite}()},
\code{\link{cer_deposit_zenodo}()},
\code{\link{cer_legislative_instrument}()},
\code{\link{cer_manifest_clear}()},
\code{\link{cer_manifest_write}()},
\code{\link{cer_sha256}()},
\code{\link{cer_snapshot}()}
}
\concept{reproducibility}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accu.R
\name{cer_accu_relinquishments}
\alias{cer_accu_relinquishments}
\title{ACCU relinquishments by project}
\source{
Clean Energy Regulator ACCU project register (CC BY 4.0).
}
\usage{
cer_accu_relinquishments(project_id = NULL)
}
\arguments{
\item{project_id}{Optional character vector of project IDs.}
}
\value{
A \code{cer_tbl} with one row per project with
\code{accus_relinquished > 0}.
}
\description{
Returns projects with non-zero ACCU relinquishments. Useful for
tracking voluntary cancellations and compliance obligations.
}
\examples{
\donttest{
op <- options(cer.cache_dir = tempdir())
cer_accu_relinquishments()
options(op)
}
}
\seealso{
Other accu: 
\code{\link{cer_accu_aggregate}()},
\code{\link{cer_accu_contracts}()},
\code{\link{cer_accu_issuances}()},
\code{\link{cer_accu_methods}()},
\code{\link{cer_accu_projects}()},
\code{\link{cer_method_determination}()},
\code{\link{cer_method_integrity}()}
}
\concept{accu}
